"""Main file"""
import sys
from PyQt5.QtWidgets import QMainWindow, QApplication
from PyQt5.QtGui import QIcon
from PyQt5.QtCore import Qt, QSettings, QTranslator, QLocale
from modules import widgets, paths


class ZVVOnlineTV(QMainWindow):
	"""Main class ZVVOnlineTV"""
	def __init__(self):
		super().__init__()
		self.create_main_windows()

	def create_main_windows(self):
		"""Create central widget"""
		self.settings = QSettings(paths.config_path, QSettings.IniFormat)
		#set central widget
		self.central_widget = widgets.Widgets()
		self.setCentralWidget(self.central_widget)

		#set main
		self.resize(1000, 600)
		self.setWindowTitle("ZVVOnlineTV")
		self.setWindowIcon(QIcon("./icons/zvvonlinetv.png"))
		self.show()

######################################################################################

	def closeEvent(self, event):
		"""Close event"""
		text = self.central_widget.pls_tab.pls_line.text()
		if text:
			saved_text = self.settings.value("config/last_pls_url")
			if text != saved_text:
				self.settings.setValue("config/last_pls_url", text)
				self.settings.sync()

		current_tab = self.central_widget.tabber.currentIndex()
		saved_tab = int(self.settings.value("config/last_tab"))
		if current_tab != saved_tab:
			self.settings.setValue("config/last_tab", current_tab)
			self.settings.sync()
		event.accept()

###############################################################

#main
APP = QApplication([])
import locale
locale.setlocale(locale.LC_NUMERIC, 'C')
SETTINGS = QSettings(paths.config_path, QSettings.IniFormat)
CURRENT_LANG = SETTINGS.value('config/language')
TRANSLATOR = QTranslator()
if CURRENT_LANG == 'system':
	lang = QLocale.system().name()
	if 'ru' in lang:
		TRANSLATOR.load('./langs/lang_rus')
	if 'uk' in lang:
		TRANSLATOR.load('./langs/lang_ukr')
if CURRENT_LANG == 'russian':
	TRANSLATOR.load('./langs/lang_rus')
if CURRENT_LANG == 'ukrainian':
	TRANSLATOR.load('./langs/lang_ukr')
APP.installTranslator(TRANSLATOR)
WIN = ZVVOnlineTV()
APP.exec_()
