from PyQt5.QtWidgets import QDialog, QWidget, QTabWidget, QVBoxLayout, QLabel, QTextEdit
from PyQt5.QtGui import QFont, QPixmap, QTextCursor
from PyQt5.QtCore import Qt, QUrl
from . import paths

class About(QDialog):
	"""About win class"""
	def __init__(self):
		super().__init__()
		self.create_about_window()

	def create_about_window(self):
		#add vbox main
		self.vbox_about = QVBoxLayout()
		self.vbox_about.setContentsMargins(0, 0, 0, 0)
		self.setLayout(self.vbox_about)

		#tabber
		self.tabber = QTabWidget()
		self.vbox_about.addWidget(self.tabber)
		
		#add tabs
		self.author_tab = Author()
		self.tabber.addTab(self.author_tab, self.tr("About"))
		#history tab
		self.history_tab = History()
		self.tabber.addTab(self.history_tab, self.tr("History"))
		#libraries tab
		self.libraries_tab = Libraries()
		self.tabber.addTab(self.libraries_tab, self.tr("Libraries"))
		
		#attribute
		self.setAttribute(Qt.WA_DeleteOnClose, True)
		
		###
		self.setWindowTitle(self.tr("About"))
		self.setModal(True)
		self.resize(500,400)
		self.show()
		self.history_tab.text_edit.moveCursor(QTextCursor.Start)
		
################################################################################

class Author(QWidget):
	"""Anchor tab class"""
	def __init__(self):
		super().__init__()
		self.create_widgets()
		
	def create_widgets(self):
		"""Create widgets"""
		self.vbox_about = QVBoxLayout()
		self.vbox_about.setContentsMargins(0, 0, 0, 0)
		self.setLayout(self.vbox_about)
		
		self.vbox_about.addStretch()
		#add image
		self.label_image = QLabel()
		self.label_image.setAlignment(Qt.AlignCenter)
		self.pixmap = QPixmap("icons/zvvonlinetv.png")
		self.pixmap = self.pixmap.scaled(100,100)
		self.label_image.setPixmap(self.pixmap)
		self.vbox_about.addWidget(self.label_image)

		#add label name_program
		self.label_program = QLabel("ZVVOnlineTV v.3.2")
		self.label_program.setAlignment(Qt.AlignCenter)
		self.label_program.setWordWrap(True)
		self.vbox_about.addWidget(self.label_program)
		#add label author
		self.label_author = QLabel(self.tr("Author: <b>Vyacheslav Zubik</b>. Ukraine, Kherson"))
		self.label_author.setAlignment(Qt.AlignCenter)
		self.label_author.setWordWrap(True)
		self.vbox_about.addWidget(self.label_author)
		#add blog
		self.label_blog = QLabel(self.tr("My blog:<a href = 'http://zvvubuntu.blogspot.com'>ZVVUbuntu`s Blog</a>"))
		self.label_blog.setAlignment(Qt.AlignCenter)
		self.label_blog.setOpenExternalLinks(True)
		self.label_blog.setWordWrap(True)
		self.vbox_about.addWidget(self.label_blog)
		#add label email
		self.label_email = QLabel("Email:  ZVVUbuntu@gmail.com")
		self.label_email.setAlignment(Qt.AlignCenter)
		self.label_email.setTextInteractionFlags(Qt.TextSelectableByMouse)
		self.label_email.setWordWrap(True)
		self.vbox_about.addWidget(self.label_email)
		#add donate label
		self.label_donate = QLabel("My game - <a href = 'https://play.google.com/store/apps/details?id=com.ZVV.ZVVMinis'><b>ZVVMinis</b></a> - mini game pack.")
		self.label_donate.setOpenExternalLinks(True)
		self.label_donate.setWordWrap(True)
		self.label_donate.setAlignment(Qt.AlignCenter)
		self.vbox_about.addWidget(self.label_donate)
		
		self.vbox_about.addStretch()
		
		
#####################################################################################################

class History(QWidget):
	"""History tab class"""
	def __init__(self):
		super().__init__()
		self.create_widgets()
		
	def create_widgets(self):
		"""Create widgets"""
		self.vbox_main = QVBoxLayout()
		self.vbox_main.setContentsMargins(0, 0, 0, 0)
		self.setLayout(self.vbox_main)
		
		font = QFont()
		font.setPointSize(10)
		self.text_edit = QTextEdit()
		self.text_edit.setReadOnly(True)
		self.text_edit.setFont(font)
		self.vbox_main.addWidget(self.text_edit)
		
		with open('./howto/history.txt', 'r', encoding='utf-8') as file_open:
			for line in file_open:
				self.text_edit.insertPlainText(line)


#####################################################################################################

class Libraries(QWidget):
	"""Libraries tab class"""
	def __init__(self):
		super().__init__()
		self.create_widgets()
		
	def create_widgets(self):
		"""Create widgets"""
		self.vbox_main = QVBoxLayout()
		self.vbox_main.setContentsMargins(0, 0, 0, 0)
		self.setLayout(self.vbox_main)
		
		#label
		self.label = QLabel()
		self.label.setWordWrap(True)
		self.label.setStyleSheet('QLabel{font-size:16px;}')
		self.vbox_main.addWidget(self.label)
		self.label.setText("""
ZVVOnlineTV uses:
			
- Python3
- PyQt5
- mpv and ffmpeg
- youtube-dl
- BeautifulSoup
- googleapiclient
- httplib2
- oauth2client
- uritemplate
""")
		
