"""BackupWin module"""
import os
import time
import zipfile
from zipfile import ZipFile as ZP
from PyQt5.QtWidgets import (QWidget, QVBoxLayout, QHBoxLayout, 
							 QLabel, QPushButton, QFileDialog, 
							 QProgressBar, QDialog)
from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtCore import Qt, QCoreApplication, QDir
from . import paths


class Backup_win(QDialog):
	"""BackupWin class"""
	def __init__(self):
		super().__init__()
		self.create_widgets()
		
	def create_widgets(self):
		"""Create widgets"""
		
	#vbox_main
		self.vbox_main = QVBoxLayout()
		self.setLayout(self.vbox_main)
		
		#main label
		self.main_label = QLabel(self.tr("Create or restore backup of your`s playlists"))
		self.main_label.setAlignment(Qt.AlignCenter)
		self.main_label.setWordWrap(True)
		self.vbox_main.addWidget(self.main_label)
		
	#hbox_center
		self.hbox_center = QHBoxLayout()
		self.vbox_main.addLayout(self.hbox_center)
		
		#button_create_backup
		icon = "./icons/create_backup.png"
		title = self.tr("Create backup")
		self.button_create_backup = Button_backup()
		self.button_create_backup.set_items(title, icon)
		self.button_create_backup.clicked.connect(self.press_create_backup_button)
		self.hbox_center.addWidget(self.button_create_backup)
		
		#
		self.hbox_center.addStretch()
		
		#button_restore_backup
		icon = "./icons/restore_backup.png"
		title = self.tr("Restore backup")
		self.button_restore_backup = Button_backup()
		self.button_restore_backup.set_items(title, icon)
		self.button_restore_backup.clicked.connect(self.press_restore_backup)
		self.hbox_center.addWidget(self.button_restore_backup)
		
		#
		self.vbox_main.addStretch()
		
		#label_info
		self.label_info = QLabel()
		self.label_info.setWordWrap(True)
		self.vbox_main.addWidget(self.label_info)
		
		#progress_bar
		self.progress_bar = QProgressBar()
		self.progress_bar.hide()
		self.vbox_main.addWidget(self.progress_bar)
		
		###
		self.resize(500, 300)
		self.setModal(True)
###############################################################

	def press_create_backup_button(self):
		"""Press create backup button"""
		pls_path = paths.path_playlists
		file_name = "backup_pls_" + time.strftime("%d-%m-%Y_%H-%M-%S") + ".zip"
		dst_file = os.path.join(QDir.homePath(), file_name)
		self.make_zipfile(dst_file, pls_path)
		
	def make_zipfile(self, output_filename, source_dir):
		"""Create zip backup"""
		self.label_info.clear()
		if not self.progress_bar.isVisible():
			self.progress_bar.show()
		self.progress_bar.setRange(0, len(source_dir) + 1)
		relroot = os.path.abspath(os.path.join(source_dir, os.pardir))
		with ZP(output_filename, "w", zipfile.ZIP_DEFLATED) as my_zip:
			for root, dirs, files in os.walk(source_dir):
				QCoreApplication.processEvents()
				# add directory (needed for empty dirs)
				my_zip.write(root, os.path.relpath(root, relroot))
				for file in files:
					QCoreApplication.processEvents()
					filename = os.path.join(root, file)
					if os.path.isfile(filename): # regular files only
						arcname = os.path.join(os.path.relpath(root, relroot), file)
						my_zip.write(filename, arcname)
					value = self.progress_bar.value()
					value += 1
					self.progress_bar.setValue(value)
		self.progress_bar.hide()
		self.label_info.setText("Backup path: {0}".format(output_filename))
		
	def press_restore_backup(self):
		"""Press resore backup"""
		file_path = QFileDialog.getOpenFileName(self, self.tr("Open backup file"), 
			QDir.homePath(), "Backups (e.g. backup_pls_name.zip)(backup_pls_*.zip)")[0]
		if os.path.exists(file_path):
			dst_path = paths.path_program
			self.restore_backup(file_path, dst_path)
			
	def restore_backup(self, output_file, source_dir):
		"""Restore backup"""
		self.label_info.clear()
		if not os.path.exists(source_dir):
			os.mkdir(source_dir)
		
		with ZP(output_file, "r", zipfile.ZIP_DEFLATED) as my_zip:
			my_zip.extractall(source_dir)
		self.label_info.setText("Backup <b>{0}</b> was restored".format(os.path.split(output_file)[1]))
###############################################################

class Button_backup(QPushButton):
	"""Button backup"""
	def __init__(self):
		super().__init__()
		self.create_widgets()
		
	def create_widgets(self):
		"""Create widgets"""
		
		#vbox_main
		self.vbox_main = QVBoxLayout()
		self.vbox_main.setContentsMargins(4, 1, 4, 1)
		self.vbox_main.setSpacing(2)
		self.setLayout(self.vbox_main)
		
		#label_name
		self.label_title = QLabel()
		self.label_title.setWordWrap(True)
		self.label_title.setAlignment(Qt.AlignCenter)
		self.label_title.setStyleSheet("QLabel{font-weight:bold;}")
		self.vbox_main.addWidget(self.label_title)
		
		#label_image
		self.label_image = QLabel()
		#self.label_image.setScaledContents(True)
		self.label_image.setAlignment(Qt.AlignCenter)
		self.vbox_main.addWidget(self.label_image)
		
		###
		self.BUTTONS_STYLE = ("""QPushButton{
						border:1px solid silver;
						border-radius:6px;
					} 
					QPushButton::hover{
						border:1px solid orange; 
						background-color:#fff19b;
					} 
					QPushButton::checked{
						border: 1px solid silver; 
						background-color:lightgreen;
					}""")
		self.setFixedSize(150, 150)
		self.setCursor(Qt.PointingHandCursor)
		self.setStyleSheet(self.BUTTONS_STYLE)
		
	def set_items(self, title, image):
		"""set button items"""
		self.label_title.setText(title)
		pixmap = QPixmap(image)
		pixmap = pixmap.scaled(100, 100, transformMode=Qt.SmoothTransformation)
		self.label_image.setPixmap(pixmap)
		
