"""Bookmarks module"""
import os
import json
from PyQt5.QtWidgets import (QWidget, QVBoxLayout, QTableWidget, QPushButton, 
							 QMenu, QTableWidgetItem, QHeaderView, QSizePolicy)
from PyQt5.QtGui import QIcon, QFont
from PyQt5.QtCore import Qt, QTimer
from . import paths


class BookmarksTab(QWidget):
	"""Bookmarks tab class"""
	def __init__(self):
		super().__init__()
		self.create_widgets()
		
	def create_widgets(self):
		"""Create bookmarks widgets"""
		#self.TIME_POSITION = 0
	#vbox_main
		self.vbox_main = QVBoxLayout()
		self.vbox_main.setContentsMargins(2, 2, 2, 2)
		self.setLayout(self.vbox_main)
		
		#table table_widget
		self.table_widget = QTableWidget(0, 1)
		self.table_widget.setHorizontalHeaderLabels((self.tr("Marks"),))
		self.table_widget.resizeRowsToContents()
		self.table_widget.verticalHeader().hide()
		self.table_widget.setShowGrid(True)
		self.table_widget.itemDoubleClicked.connect(self.press_bookmarks_item)
		self.vbox_main.addWidget(self.table_widget)
		self.table_widget.horizontalHeader().setSectionResizeMode(0, QHeaderView.Stretch)
		
		#menu button
		self.button_menu = QPushButton(QIcon(paths.menu_icon), self.tr("Menu"))
		self.button_menu.setStyleSheet(paths.BUTTONS_STYLE)
		self.button_menu.setMinimumHeight(30)
		self.button_menu.setCursor(Qt.PointingHandCursor)
		self.button_menu.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
		self.vbox_main.addWidget(self.button_menu)
		#menu
		font = QFont()
		font.setPointSize(12)
		self.menu = QMenu()
		self.menu.setFont(font)
		self.menu.addAction(QIcon(paths.remove_icon), self.tr("Remove item"), self.press_remove_bookmarks)
		self.button_menu.setMenu(self.menu)
		
		###autostart
		self.set_bookmarks()
##########################################################################

	def set_bookmarks(self):
		"""Set bookmarks"""
		bookmarks_path = paths.bookmarks_path
		list_bookmarks = os.listdir(bookmarks_path)
		for item in list_bookmarks:
			bookmarks_full_path = os.path.join(bookmarks_path, item)
			if "json" in bookmarks_full_path:
				if os.path.exists(bookmarks_full_path):
					with open(bookmarks_full_path) as file_load:
						info = json.load(file_load)
						url = info[0]
						position = int(info[1])
					self.add_one_row(item, url, bookmarks_full_path, position)
				
	def add_one_row(self, name, url, file_path, position):
		"""Add one row"""
		#name
		fileNameItem = MyTableWidgetItem()
		fileNameItem.setIcon(QIcon(paths.bookmarks_icon))
		fileNameItem.setText(name)
		fileNameItem.setToolTip(name)
		fileNameItem.URL = url
		fileNameItem.SAVE_POSITION = position
		fileNameItem.FILE_PATH = file_path
		fileNameItem.setFlags(fileNameItem.flags() ^ Qt.ItemIsEditable)
		fileNameItem.setTextAlignment(Qt.AlignVCenter | Qt.AlignLeft)
		#set
		row = self.table_widget.rowCount()
		self.table_widget.insertRow(row)
		self.table_widget.setItem(row, 0, fileNameItem)

	def press_remove_bookmarks(self):
		"""Press remove bookmarks"""
		selected_items = self.table_widget.selectedItems()
		if len(selected_items) > 0:
			for current_item in selected_items:
				name = current_item.text()
				file_path = current_item.FILE_PATH
				if os.path.exists(file_path):
					os.remove(file_path)
				self.table_widget.removeRow(current_item.row())
		
	def press_bookmarks_item(self, item):
		"""Press bookmarks item"""
		url = item.URL
		position = item.SAVE_POSITION
		name = item.text().replace(".json", "")
		self.parent().parent().parent().press_any_item(name, url)
		self.parent().parent().parent().rewind(position)
		
	def update_items(self):
		"""Update items"""
		self.table_widget.setRowCount(0)
		self.set_bookmarks()
		
###########################################################################

class MyTableWidgetItem(QTableWidgetItem):
	"""MyTableWidgetItem"""
	def __init__(self):
		super().__init__()
		self.URL = None
		self.SAVE_POSITION = None
		self.FILE_PATH = None
