"""Config tab module"""
import os
import sys
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from . import (add_station, new_pls, paths, howtoWin, about)

class Config(QWidget):
	"""Config Tab class"""
	def __init__(self):
		super().__init__()
		self.create_widgets()

	def create_widgets(self):
		"""Create widgets"""
		self.settings = QSettings(paths.config_path, QSettings.IniFormat)
		
		#vbox_config
		self.vbox_config = QVBoxLayout()
		self.vbox_config.setContentsMargins(0, 0, 0, 0)
		self.setLayout(self.vbox_config)
		
		#scrollarea
		self.scroll_area = QScrollArea()
		self.scroll_area.setWidgetResizable(True)
		self.scroll_area.setStyleSheet("QScrollArea{border:1px solid transparent;}" + paths.get_scrollbar_style())
		self.vbox_config.addWidget(self.scroll_area)
		
		#widget_config
		self.widget_config = QWidget()
		self.scroll_area.setWidget(self.widget_config)
		
		#vbox_main
		self.vbox_main = QVBoxLayout()
		self.vbox_main.setContentsMargins(0, 0, 0, 0)
		self.widget_config.setLayout(self.vbox_main)
		
	#add hbox check stream
		self.hbox_check_channel = QHBoxLayout()
		self.vbox_main.addLayout(self.hbox_check_channel)
		#add entry
		self.entry_check = QLineEdit()
		self.entry_check.setClearButtonEnabled(True)
		self.entry_check.setPlaceholderText(self.tr("Put a stream"))
		self.entry_check.setMinimumHeight(30)
		self.entry_check.returnPressed.connect(self.stream_line_pressed)
		self.hbox_check_channel.addWidget(self.entry_check)
		#add button check
		self.button_check = QPushButton()
		self.button_check.setMinimumHeight(30)
		self.button_check.setMinimumWidth(30)
		icon = QIcon("icons/check_icon.png")
		self.button_check.setIconSize(QSize(25, 25))
		self.button_check.setIcon(icon)
		self.button_check.setCursor(Qt.PointingHandCursor)
		self.button_check.setStyleSheet(paths.BUTTONS_STYLE)
		self.hbox_check_channel.addWidget(self.button_check)
		
	#add button add
		self.button_add_to_base = QPushButton("Add to Base")
		self.button_add_to_base.hide()
		self.button_add_to_base.clicked.connect(self.press_button_add_to_base)
		self.vbox_main.addWidget(self.button_add_to_base)

	#add hbox_jump_time
		self.hbox_jump_time = QHBoxLayout()
		self.vbox_main.addLayout(self.hbox_jump_time)
		#label
		self.label_jump_time = QLabel(self.tr("TimeJump (in sec):"))
		self.hbox_jump_time.addWidget(self.label_jump_time)
		#spinbox
		self.spinbox_jump_time = QSpinBox()
		self.spinbox_jump_time.setMaximum(9999)
		self.hbox_jump_time.addWidget(self.spinbox_jump_time)
		
	#add combo for change lang
		main_font = QFont()
		main_font.setPointSize(10)
		self.hbox_langs = QHBoxLayout()
		self.vbox_main.addLayout(self.hbox_langs)
		#add label
		self.label_langs = QLabel(self.tr("Change language:"))
		self.label_langs.setFont(main_font)
		self.hbox_langs.addWidget(self.label_langs)
		#add combobox
		self.combobox_langs = QComboBox()
		self.hbox_langs.addWidget(self.combobox_langs)
		list_langs = [self.tr('Choose:'),'System','Eng','Rus','Ukr',]
		for item in list_langs:
			self.combobox_langs.addItem(item)
		self.combobox_langs.activated.connect(self.change_language)

	#add timer hbox
		self.hbox_timer = QHBoxLayout()
		self.vbox_main.addLayout(self.hbox_timer)
		#label
		self.label_timer = QLabel(self.tr("Timer:"))
		self.hbox_timer.addWidget(self.label_timer)
		#combo_time
		self.combo_time = QComboBox()
		self.hbox_timer.addWidget(self.combo_time)

		self.time_list = [self.tr("Set min:"),"5","10","15","30","45","60","90","120","180",]
		for item in self.time_list:
			self.combo_time.addItem(item)
		self.combo_time.activated.connect(self.set_time)

		#after time
		self.combo_what_do = QComboBox()
		self.hbox_timer.addWidget(self.combo_what_do)
		self.do_list = [
						self.tr("After:"),
						self.tr("Close"),
						self.tr("Suspend"),
						self.tr("Shutdown"),
					]
		for item in self.do_list:
			self.combo_what_do.addItem(item)

		####timer
		self.timer = QTimer()
		self.timer.setInterval(60000)
		self.timer.timeout.connect(self.change_time)

		#add progress bar
		self.progress_bar = QProgressBar()
		self.progress_bar.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
		self.progress_bar.hide()
		self.vbox_main.addWidget(self.progress_bar)

		#add video options
		self.button_video_options = QPushButton(self.tr("***Adjust video***"))
		self.button_video_options.setStyleSheet(paths.BUTTONS_STYLE)
		self.button_video_options.setMinimumHeight(30)
		self.button_video_options.setCursor(Qt.PointingHandCursor)
		self.button_video_options.clicked.connect(self.press_video_options)
		self.vbox_main.addWidget(self.button_video_options)

		self.create_video_options()

		self.vbox_main.addStretch()
		
		self.create_info_widget()
		
	#hbox_about_guide
		self.hbox_about_guide = QHBoxLayout()
		self.vbox_config.addLayout(self.hbox_about_guide)
		#add push how to
		self.howto_button = QPushButton(self.tr("How to"))
		self.howto_button.setStyleSheet(paths.BUTTONS_STYLE)
		self.howto_button.setCursor(Qt.PointingHandCursor)
		self.howto_button.setMinimumHeight(35)
		self.howto_button.setIcon(QIcon(paths.howto_icon))
		self.howto_button.clicked.connect(self.press_howto_button)
		self.hbox_about_guide.addWidget(self.howto_button)
		#button_how_video
		self.button_how_video = QPushButton()
		self.button_how_video.setStyleSheet(paths.BUTTONS_STYLE)
		self.button_how_video.setCursor(Qt.PointingHandCursor)
		self.button_how_video.setFixedSize(35, 35)
		self.button_how_video.setIconSize(QSize(25, 25))
		self.button_how_video.setIcon(QIcon(paths.yt_icon))
		self.button_how_video.clicked.connect(self.press_howto_video_button)
		self.hbox_about_guide.addWidget(self.button_how_video)
		#button about
		self.button_about = QPushButton()
		self.button_about.setIcon(QIcon(paths.about_icon))
		self.button_about.setFixedSize(35, 35)
		self.button_about.setIconSize(QSize(25, 25))
		self.button_about.setCursor(Qt.PointingHandCursor)
		self.button_about.setStyleSheet(paths.BUTTONS_STYLE)
		self.button_about.clicked.connect(self.press_about_button)
		self.hbox_about_guide.addWidget(self.button_about)
		
##################################################################################################

	def press_about_button(self):
		"""Create about win"""
		self.about_win = about.About()

	def press_add_tabs_button(self):
		"""Press add tabs button"""
		if self.tabs_widget.isVisible():
			self.tabs_widget.hide()
		else:
			self.tabs_widget.show()
		
	def stream_line_pressed(self):
		"""Stream line pressed"""
		text = self.entry_check.text()
		if text == "zvvubuntu_show_me_a_hide_button":
			self.button_add_to_base.show()

	def press_button_add_to_base(self):
		"""Press button add to base"""
		from . import addchbase
		self.add_chbase = addchbase.AddCh()
		
	def press_howto_video_button(self):
		"""Press how to video button"""
		url = "https://www.youtube.com/watch?v=Qj2j8ORApvc"
		title = "ZVVOnlineTV"
		self.parent().parent().parent().press_any_item(title, url)

	def press_howto_button(self):
		self.howto_win = howtoWin.HowtoWin()
		self.howto_win.show()

	def press_create_pls_button(self):
		"""Press create pls button"""
		self.create_win = new_pls.New_pls()

	def press_video_options(self):
		"""Press video options"""
		if self.video_options_widget.isVisible():
			self.video_options_widget.hide()
		else:
			self.video_options_widget.show()

	def create_video_options(self):
		"""Create video options"""
		self.video_options_widget = QFrame()
		self.video_options_widget.setFrameShape(QFrame.Box)
		self.video_options_widget.setLineWidth(1)
		self.video_options_widget.setFrameShadow(QFrame.Raised)
		self.video_options_widget.hide()
		self.vbox_main.addWidget(self.video_options_widget)
		self.vbox_v_options = QVBoxLayout()
		self.video_options_widget.setLayout(self.vbox_v_options)
		#*		BRIGHTNESS
		self.label_brightness_main = QLabel(self.tr("Brightness"))
		self.label_brightness_main.setAlignment(Qt.AlignCenter)
		self.vbox_v_options.addWidget(self.label_brightness_main)
		#brightness
		self.hbox_brightness = QHBoxLayout()
		self.vbox_v_options.addLayout(self.hbox_brightness)
		self.slider_brightness = QSlider(Qt.Horizontal)
		self.slider_brightness.setRange(-100, 100)
		self.hbox_brightness.addWidget(self.slider_brightness)
		self.label_brightness = QLabel()
		self.hbox_brightness.addWidget(self.label_brightness)
		#**		CONTRAST
		self.label_contrast_main = QLabel(self.tr("Contrast"))
		self.label_contrast_main.setAlignment(Qt.AlignCenter)
		self.vbox_v_options.addWidget(self.label_contrast_main)
		#contrast
		self.hbox_contrast = QHBoxLayout()
		self.vbox_v_options.addLayout(self.hbox_contrast)
		self.slider_contrast = QSlider(Qt.Horizontal)
		self.slider_contrast.setRange(-100, 100)
		self.hbox_contrast.addWidget(self.slider_contrast)
		self.label_contrast = QLabel()
		self.hbox_contrast.addWidget(self.label_contrast)
		#***		SATURATION
		self.label_saturation_main = QLabel(self.tr("Saturation"))
		self.label_saturation_main.setAlignment(Qt.AlignCenter)
		self.vbox_v_options.addWidget(self.label_saturation_main)
		#SAT
		self.hbox_saturation = QHBoxLayout()
		self.vbox_v_options.addLayout(self.hbox_saturation)
		self.slider_saturation = QSlider(Qt.Horizontal)
		self.slider_saturation.setRange(-100, 100)
		self.hbox_saturation.addWidget(self.slider_saturation)
		self.label_saturation = QLabel()
		self.hbox_saturation.addWidget(self.label_saturation)
		#***		HUE
		self.label_hue_main = QLabel(self.tr("Hue"))
		self.label_hue_main.setAlignment(Qt.AlignCenter)
		self.vbox_v_options.addWidget(self.label_hue_main)
		#hue
		self.hbox_hue = QHBoxLayout()
		self.vbox_v_options.addLayout(self.hbox_hue)
		self.slider_hue = QSlider(Qt.Horizontal)
		self.slider_hue.setRange(-100, 100)
		self.hbox_hue.addWidget(self.slider_hue)
		self.label_hue = QLabel()
		self.hbox_hue.addWidget(self.label_hue)
		#***		default
		self.hbox_default_hbox = QHBoxLayout()
		self.vbox_v_options.addLayout(self.hbox_default_hbox)
		#stretch
		self.hbox_default_hbox.addStretch()
		#default button
		self.default_button = QPushButton(self.tr(" Default "))
		self.default_button.setCursor(Qt.PointingHandCursor)
		self.default_button.setStyleSheet(paths.BUTTONS_STYLE)
		self.default_button.setMinimumHeight(25)
		self.hbox_default_hbox.addWidget(self.default_button)
		
		
	def set_brightness_text(self, value):
		self.label_brightness.setText(str(value))

	def set_contrast_text(self, value):
		self.label_contrast.setText(str(value))

	def set_saturation_text(self, value):
		self.label_saturation.setText(str(value))

	def set_hue_text(self, value):
		self.label_hue.setText(str(value))

	def default_values(self):
		value = 0
		self.slider_brightness.setValue(value)
		self.slider_contrast.setValue(value)
		self.slider_saturation.setValue(value)
		self.slider_hue.setValue(value)

	def change_time(self):
		"""Change time"""
		value = self.progress_bar.value()
		value -= 1
		self.progress_bar.setValue(value)
		self.progress_bar.setFormat(str(value) + self.tr(" min"))
		if value <= 0:
			self.timer.stop()
			if self.combo_what_do.currentIndex() == 1:
				QApplication.exit()
			if self.combo_what_do.currentIndex() == 2:
				try:
					QProcess.startDetached("systemctl suspend")
				except:
					pass
			if self.combo_what_do.currentIndex() == 3:
				if sys.platform.startswith("win"):
					QProcess.startDetached("shutdown.exe /s")
				if sys.platform.startswith("linux"):
					QProcess.startDetached("shutdown -P now")

	def set_time(self):
		"""Timer check"""
		if self.combo_time.currentIndex() > 0:
			self.timer.stop()
			time = int(self.combo_time.currentText())
			self.start_progress(time)
			self.timer.start()
		if self.combo_time.currentIndex() == 0:
			self.timer.stop()
			self.progress_bar.hide()

	def start_progress(self, time):
		self.progress_bar.show()
		self.progress_bar.setRange(0, time)
		self.progress_bar.setValue(time)
		self.progress_bar.setFormat(str(time) + self.tr(" min"))

	def press_add_button(self):
		"""Press add win button"""
		self.win_add = add_station.Add_Station()
		
	def change_language(self):
		current_item = self.combobox_langs.currentIndex()
		if current_item == 1:
			self.settings.setValue('config/language','system')
		if current_item == 2:
			self.settings.setValue('config/language','english')
		if current_item == 3:
			self.settings.setValue('config/language','russian')
		if current_item == 4:
			self.settings.setValue('config/language','ukrainian')
		self.settings.sync()
		if not self.widget_info.isVisible():
			self.widget_info.show()
			
	def create_info_widget(self):
		"""Create info widget"""
		self.widget_info = QWidget()
		self.widget_info.hide()
		self.widget_info.setStyleSheet("background:lightblue; border:1px solid silver; border-radius:5px;")
		self.vbox_main.addWidget(self.widget_info)
		self.hbox_info = QHBoxLayout()
		self.widget_info.setLayout(self.hbox_info)
		#label
		self.label_info = QLabel(self.tr("Restart is needed"))
		self.label_info.setStyleSheet("QLabel{border:1px solid transparent;}")
		self.label_info.setAlignment(Qt.AlignCenter)
		self.label_info.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
		self.hbox_info.addWidget(self.label_info)
		#
		#button_info_close
		self.button_info_close = QPushButton()
		self.button_info_close.setFlat(True)
		self.button_info_close.setIcon(QIcon(paths.remove_icon))
		self.button_info_close.setCursor(Qt.PointingHandCursor)
		self.button_info_close.setStyleSheet("""
											QPushButton{border:1px solid transparent;}
											QPushButton::hover{
												background:#ffadad;
											}""")
		self.button_info_close.clicked.connect(self.press_button_info_close)
		self.hbox_info.addWidget(self.button_info_close)
		
	def press_button_info_close(self):
		"""press_button_info_close"""
		if self.widget_info.isVisible():
			self.widget_info.hide()
		
