import os
import json
from PyQt5.QtWidgets import (QWidget, QVBoxLayout, QHBoxLayout, QComboBox, QMenu,
							 QPushButton, QLabel, QSlider, QButtonGroup, QSizePolicy)
from PyQt5.QtGui import QIcon, QFont
from PyQt5.QtCore import Qt, QSettings, QSize
from . import paths, zslider


class ControlWidget(QWidget):
	def __init__(self):
		super().__init__()
		self.create_widgets()
		
	def create_widgets(self):
		self.config_file = QSettings(paths.config_path, QSettings.IniFormat)
		MY_BUTTON_SIZE = paths.BUTTON_SIZE
		time_font = QFont()
		time_font.setPointSize(8)

	#vbox controls
		self.vbox_controls = QVBoxLayout()
		self.vbox_controls.setContentsMargins(0, 0, 0, 1)
		self.setLayout(self.vbox_controls)
	#position widget
		self.position_widget = QWidget()
		self.vbox_controls.addWidget(self.position_widget)
	#hbox position
		self.hbox_time_position = QHBoxLayout()
		self.hbox_time_position.setContentsMargins(1, 1, 1, 1)
		self.hbox_time_position.setSpacing(2)
		self.position_widget.setLayout(self.hbox_time_position)
		
		#prev button
		self.button_prev = QPushButton()
		icon = QIcon("icons/prev.png")
		self.button_prev.setIconSize(QSize(MY_BUTTON_SIZE-8, MY_BUTTON_SIZE-8))
		self.button_prev.setIcon(icon)
		self.button_prev.setCursor(Qt.PointingHandCursor)
		self.button_prev.setFixedSize(MY_BUTTON_SIZE, MY_BUTTON_SIZE)
		self.button_prev.setStyleSheet(paths.BUTTONS_STYLE)
		self.hbox_time_position.addWidget(self.button_prev)
		#button_minus_time
		self.button_minus_time = QPushButton("-10")
		self.button_minus_time.setFont(time_font)
		self.button_minus_time.setCursor(Qt.PointingHandCursor)
		self.button_minus_time.setFixedSize(MY_BUTTON_SIZE, MY_BUTTON_SIZE)
		self.button_minus_time.setStyleSheet(paths.BUTTONS_STYLE)
		self.hbox_time_position.addWidget(self.button_minus_time)
		#... position_slider
		self.vbox_position = QVBoxLayout()
		self.hbox_time_position.addLayout(self.vbox_position)
		#.hbox
		self.hbox_pos_labels = QHBoxLayout()
		self.vbox_position.addLayout(self.hbox_pos_labels)
		#.l current
		self.label_pos_current = QLabel()
		self.label_pos_current.setStyleSheet("color:green; font-weight:bold;")
		self.hbox_pos_labels.addWidget(self.label_pos_current)
		#stretch
		self.hbox_pos_labels.addStretch()
		#move time
		self.label_move_time = QLabel()
		self.label_move_time.setStyleSheet("color:blue;")
		self.label_move_time.hide()
		self.hbox_pos_labels.addWidget(self.label_move_time)
		#stretch
		self.hbox_pos_labels.addStretch()
		#.l last
		self.label_pos_last = QLabel()
		self.label_pos_last.setStyleSheet("color:red; font-weight:bold;")
		self.hbox_pos_labels.addWidget(self.label_pos_last)
		#..
		self.position_slider = QSlider(Qt.Horizontal)
		self.position_slider.setStyleSheet(self.get_slider_style())
		self.vbox_position.addWidget(self.position_slider)
		
		#button_plus_time
		self.button_plus_time = QPushButton("+15")
		self.button_plus_time.setFont(time_font)
		self.button_plus_time.setCursor(Qt.PointingHandCursor)
		self.button_plus_time.setFixedSize(MY_BUTTON_SIZE, MY_BUTTON_SIZE)
		self.button_plus_time.setStyleSheet(paths.BUTTONS_STYLE)
		self.hbox_time_position.addWidget(self.button_plus_time)
		#next button
		self.button_next = QPushButton()
		icon = QIcon("icons/next2.png")
		self.button_next.setIconSize(QSize(MY_BUTTON_SIZE-8, MY_BUTTON_SIZE-8))
		self.button_next.setIcon(icon)
		self.button_next.setCursor(Qt.PointingHandCursor)
		self.button_next.setFixedSize(MY_BUTTON_SIZE, MY_BUTTON_SIZE)
		self.button_next.setStyleSheet(paths.BUTTONS_STYLE)
		self.hbox_time_position.addWidget(self.button_next)
		
	#control hbox
		self.hbox_controls = QHBoxLayout()
		self.hbox_controls.setContentsMargins(0,0,0,0)
		self.vbox_controls.addLayout(self.hbox_controls)
		#.
		#button hide stations
		self.button_hide_left = QPushButton()
		icon = QIcon("icons/hide_icon2.png")
		self.style_button(self.button_hide_left, icon)
		self.hbox_controls.addWidget(self.button_hide_left)
		
		#button_play
		self.button_play = QPushButton()
		icon = QIcon(paths.play_icon)
		self.style_button(self.button_play, icon)
		self.hbox_controls.addWidget(self.button_play)
		#..
		self.button_stop = QPushButton()
		icon = QIcon(paths.stop_icon)
		self.style_button(self.button_stop, icon)
		self.hbox_controls.addWidget(self.button_stop)

		#...pause
		self.button_pause = QPushButton()
		icon = QIcon(paths.pause_icon)
		self.style_button(self.button_pause, icon)
		##self.button_pause.clicked.connect(self.press_button_pause)
		self.hbox_controls.addWidget(self.button_pause)

		#...
		#add vbox volume and label
		self.vbox_volume = QVBoxLayout()
		self.vbox_volume.setContentsMargins(0, 0, 0, 0)
		self.vbox_volume.setSpacing(0)
		#add volume
		self.volume_slider = zslider.ZSlider()
		self.volume_slider.setValue(int(self.config_file.value('config/volume')))
		self.volume_slider.setRange(0,100)
		##self.volume_slider.valueChanged.connect(self.volume_change)
		self.volume_slider.setMaximumWidth(130)
		#add label volume
		self.hbox_volume_info = QHBoxLayout()
		self.hbox_volume_info.setContentsMargins(0,0,0,0)
		self.vbox_volume.addLayout(self.hbox_volume_info)
		self.label_volume = QLabel()
		self.label_volume.setAlignment(Qt.AlignCenter)
		self.label_volume.setText(self.tr("Volume:") + str(self.volume_slider.sliderPosition()))
		self.hbox_volume_info.addWidget(self.label_volume)
		#mute button
		self.button_mute = QPushButton()
		self.button_mute.setFlat(True)
		self.button_mute.setFixedSize(20,20)
		icon = QIcon(paths.volume_icon)
		self.button_mute.setIcon(icon)
		self.button_mute.setIconSize(QSize(17,17))
		self.button_mute.setCursor(Qt.PointingHandCursor)
		self.hbox_volume_info.addWidget(self.button_mute)
		#
		self.vbox_volume.addWidget(self.volume_slider)
		self.hbox_controls.addLayout(self.vbox_volume)
		
	#button tvgid
		self.button_tvgid = QPushButton()
		icon = QIcon(paths.tvgid_icon)
		self.style_button(self.button_tvgid, icon)
		self.hbox_controls.addWidget(self.button_tvgid)
		
		#
		self.hbox_controls.addStretch()
		
	#time_save button
		self.button_time_save = QPushButton()
		icon = QIcon("icons/time_clock_icon.png")
		self.style_button(self.button_time_save, icon)
		self.hbox_controls.addWidget(self.button_time_save)
		#time_clock_menu
		menu_font = QFont()
		menu_font.setPointSize(12)
		self.menu_time_save = QMenu()
		self.menu_time_save.setFont(menu_font)
		self.menu_time_save.addAction(QIcon("icons/bookmark.png"), self.tr("Add mark"), self.press_save_time_button)
		self.menu_time_save.addSeparator()
		self.button_time_save.setMenu(self.menu_time_save)
		
	#modes combo
		self.combo_playmodes = QComboBox()
		self.combo_playmodes.setMaximumHeight(paths.BUTTON_SIZE)
		self.combo_playmodes.setFocusPolicy(Qt.NoFocus)
		self.combo_playmodes.addItem(QIcon(paths.stop_mode), "Stop")
		self.combo_playmodes.addItem(QIcon(paths.replay_mode), "Replay")
		self.combo_playmodes.addItem(QIcon(paths.next_mode), "Next")
		self.combo_playmodes.addItem(QIcon(paths.shuffle_mode), "Shuffle")
		self.hbox_controls.addWidget(self.combo_playmodes)
		
		#button fullscreen
		self.button_fullscreen = QPushButton()
		icon = QIcon(paths.fullscreen_icon)
		self.style_button(self.button_fullscreen, icon)
		self.hbox_controls.addWidget(self.button_fullscreen)
		
####################################################################################

	def style_button(self, button, icon):
		"""Styled button"""
		button.setIconSize(QSize(paths.BUTTON_SIZE-4, paths.BUTTON_SIZE-4))
		button.setIcon(icon)
		button.setMinimumWidth(paths.BUTTON_SIZE)
		button.setMaximumWidth(paths.BUTTON_SIZE)
		button.setMinimumHeight(paths.BUTTON_SIZE)
		button.setMaximumHeight(paths.BUTTON_SIZE)
		button.setCursor(Qt.PointingHandCursor)
		button.setStyleSheet(paths.BUTTONS_STYLE)
		
	def press_save_time_button(self):
		"""Press save time button"""
		if self.parent().parent().player._get_property('duration'):
			if self.parent().parent().player._get_property('duration') > 0:
				position = self.parent().parent().player._get_property('time-pos')
				name = self.parent().parent().errors_label.text()
				bad_letters = ":/\\[]()&*^%$:\"|!@~`'"
				normal_file_rename = ''.join(l for l in name if l not in bad_letters)
				bookmarks_file = "{0}.json".format(normal_file_rename)
				bookmarks_full_path = os.path.join(paths.bookmarks_path, bookmarks_file)
				with open(bookmarks_full_path, 'w') as file_save:
					json.dump([self.parent().parent().CURRENT_URL, position], file_save)
				self.parent().parent().bookmarks_tab.update_items()

	def get_slider_style(self):
		"""Get slider style"""
		PROGRESS_SLIDER_STYLE = ("""
		QSlider::groove:horizontal {
			border: 1px solid #bbb;
			background: white;
			height: 8px;
			border-radius: 4px;
		}
		QSlider::sub-page:horizontal {
		background: qlineargradient(x1: 0, y1: 0,    x2: 0, y2: 1,
		    stop: 0 #66e, stop: 1 #bbf);
		background: qlineargradient(x1: 0, y1: 0.2, x2: 1, y2: 1,
		    stop: 0 #baffa3, stop: 1 #8aff63);
		border: 1px solid #777;
		height: 10px;
		border-radius: 4px;
		}
		QSlider::add-page:horizontal {
		background: #fff;
		border: 1px solid #777;
		height: 10px;
		border-radius: 4px;
		}
		QSlider::handle:horizontal {
		background: qlineargradient(x1:0, y1:0, x2:1, y2:1,
		    stop:0 #eee, stop:1 #ccc);
		border: 1px solid #777;
		width: 13px;
		margin-top: -2px;
		margin-bottom: -2px;
		border-radius: 4px;
		}
		QSlider::handle:horizontal:hover {
		background: qlineargradient(x1:0, y1:0, x2:1, y2:1,
		    stop:0 #fff, stop:1 #ddd);
		border: 1px solid #444;
		border-radius: 4px;
		}
		QSlider::sub-page:horizontal:disabled {
		background: #bbb;
		border-color: #999;
		}
		QSlider::add-page:horizontal:disabled {
		background: #eee;
		border-color: #999;
		}
		QSlider::handle:horizontal:disabled {
		background: #eee;
		border: 1px solid #aaa;
		border-radius: 4px;
		}
		""")
		return PROGRESS_SLIDER_STYLE
