"""Edit window module"""
import os
import sys
from PyQt5.QtWidgets import QDialog, QVBoxLayout, QHBoxLayout, QLabel, QLineEdit, QPushButton, QFileDialog
from PyQt5.QtGui import QIcon
from PyQt5.QtCore import Qt, QDir
from . import paths


class Edit(QDialog):
	"""Edit class"""
	def __init__(self, name, url, tableWidgetItem):
		super().__init__()
		self.name = name
		self.url = url
		self.tableWidgetItem = tableWidgetItem
		self.create_edit_widgets()

	def create_edit_widgets(self):
		"""Create widgets"""
		self.vbox_edit = QVBoxLayout()
		self.setLayout(self.vbox_edit)
	
	#add hbox_name_station
		self.hbox_name_station = QHBoxLayout()
		self.vbox_edit.addLayout(self.hbox_name_station)
		#add label
		self.label_name = QLabel(self.tr("Name: \t"))
		self.hbox_name_station.addWidget(self.label_name)
		#add line
		self.line_name = QLineEdit()
		self.line_name.setText(self.name)
		self.line_name.setEnabled(False)
		self.hbox_name_station.addWidget(self.line_name)

	#add hbox_stream_station
		self.hbox_stream_station = QHBoxLayout()
		self.vbox_edit.addLayout(self.hbox_stream_station)
		#add label
		self.label_stream = QLabel(self.tr("Change url: \t"))
		self.hbox_stream_station.addWidget(self.label_stream)
		#add line
		self.line_stream = QLineEdit()
		self.line_stream.setText(self.url)
		self.line_stream.setEnabled(False)
		self.hbox_stream_station.addWidget(self.line_stream)
	#add local file chooser
		self.hbox_filechooser = QHBoxLayout()
		self.vbox_edit.addLayout(self.hbox_filechooser)
		#add label
		self.label_filechooser = QLabel(self.tr("Choose file: \t"))
		self.hbox_filechooser.addWidget(self.label_filechooser)
		#add line
		self.button_filechooser = QPushButton()
		self.button_filechooser.setFixedSize(25,25)
		self.button_filechooser.setIcon(QIcon(paths.open_icon))
		self.button_filechooser.setEnabled(False)
		self.button_filechooser.clicked.connect(self.press_filechooser_icon)
		self.hbox_filechooser.addWidget(self.button_filechooser)
		
	#add stretch
		self.vbox_edit.addStretch()

	#add hbox button unlock and save
		self.hbox_unlock_save = QHBoxLayout()
		self.vbox_edit.addLayout(self.hbox_unlock_save)
		#add button unlock
		self.button_unlock = QPushButton(QIcon(paths.lock_icon), self.tr("Unlock"))
		self.button_unlock.setStyleSheet(paths.BUTTONS_STYLE)
		self.button_unlock.setMinimumHeight(30)
		self.button_unlock.setCursor(Qt.PointingHandCursor)
		self.button_unlock.clicked.connect(self.press_unlock_button)
		self.hbox_unlock_save.addWidget(self.button_unlock)
		#add button save
		self.button_save = QPushButton(QIcon(paths.save_icon), self.tr("Save"))
		self.button_save.setEnabled(False)
		self.button_save.setMinimumHeight(30)
		self.button_save.setStyleSheet(paths.BUTTONS_STYLE)
		self.button_save.setCursor(Qt.PointingHandCursor)
		self.button_save.clicked.connect(self.press_save_button)
		self.hbox_unlock_save.addWidget(self.button_save)
		
		#set
		self.setWindowTitle(self.tr("Edit station"))
		self.setModal(True)
		self.setFixedSize(600, 300)

##############################################################################################

	def press_filechooser_icon(self):
		"""Press file chooser"""
		local_path = QFileDialog.getOpenFileName(self, "Choose file", QDir.homePath(), "Video Files " + paths.videos_reg_exp)
		if local_path[0]:
			local_path = local_path[0]
			if os.path.exists(local_path):
				self.line_stream.setText(local_path)

	def press_unlock_button(self):
		"""Press unlock button"""
		self.line_name.setEnabled(True)
		self.line_stream.setEnabled(True)
		self.button_save.setEnabled(True)
		self.button_filechooser.setEnabled(True)
		self.button_unlock.setEnabled(False)

	def press_save_button(self):
		"""Press save button"""
		new_url = self.line_stream.text()
		if new_url:
			new_name = self.line_name.text().strip()
			update_params = (new_url, new_name, self.url, self.name)
			paths.BASE_CURSOR.execute("UPDATE favs SET zurl=?, zname=? WHERE zurl=? AND zname=?", update_params)
			paths.BASE_CONNECTION.commit()
			#clear widgets
			self.line_name.setEnabled(False)
			self.line_stream.setText(new_url)
			self.line_stream.setEnabled(False)
			self.button_save.setEnabled(False)
			self.button_filechooser.setEnabled(False)
			#
			self.button_unlock.setEnabled(True)
			#change url and name
			self.tableWidgetItem.setText(new_name)
			self.tableWidgetItem.URL = new_url
