"""FavTab module"""
import os
from PyQt5.QtWidgets import *
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from . import paths, edit_window, add_station


class FavTab(QWidget):
	"""Fav Tab class"""
	def __init__(self):
		super().__init__()
		self.create_widgets()

	def create_widgets(self):
		"""Create widgets"""
		self.vbox_main = QVBoxLayout()
		self.vbox_main.setContentsMargins(3, 3, 3, 3)
		self.setLayout(self.vbox_main)

		#table view
		self.table_view = QTableWidget(0, 1)
		self.table_view.setHorizontalHeaderLabels((self.tr("Name"),))
		self.table_view.resizeRowsToContents()
		self.table_view.verticalHeader().hide()
		self.table_view.horizontalHeader().hide()
		self.table_view.setShowGrid(True)
		self.table_view.itemDoubleClicked.connect(self.press_fav_item)
		self.table_view.verticalScrollBar().setStyleSheet(paths.get_scrollbar_style())
		self.vbox_main.addWidget(self.table_view)
		self.table_view.horizontalHeader().setSectionResizeMode(0, QHeaderView.Stretch)

	#hbox_remove_edit
		self.hbox_remove_edit = QHBoxLayout()
		self.vbox_main.addLayout(self.hbox_remove_edit)
		#add button
		self.add_button = QPushButton()
		self.add_button.setMinimumHeight(30)
		self.add_button.setMinimumWidth(30)
		self.add_button.setIconSize(QSize(25,25))
		self.add_button.setIcon(QIcon(paths.add_channel_icon))
		self.add_button.setCursor(Qt.PointingHandCursor)
		self.add_button.setStyleSheet(paths.BUTTONS_STYLE)
		self.add_button.clicked.connect(self.press_add_button)
		self.hbox_remove_edit.addWidget(self.add_button)
		#remove button
		self.button_menu = QPushButton(QIcon(paths.menu_icon), self.tr("Menu"))
		self.button_menu.setStyleSheet(paths.BUTTONS_STYLE)
		self.button_menu.setMinimumHeight(30)
		self.button_menu.setCursor(Qt.PointingHandCursor)
		self.button_menu.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
		#self.button_remove.clicked.connect(self.press_remove_fav)
		self.hbox_remove_edit.addWidget(self.button_menu)
		#remove menu
		font = QFont()
		font.setPointSize(12)
		self.fav_menu = QMenu()
		self.fav_menu.setFont(font)
		self.fav_menu.addAction(QIcon(paths.save_icon), self.tr("Save All as PLS"), self.press_save_favs)
		self.fav_menu.addSeparator()
		self.fav_menu.addAction(QIcon(paths.remove_icon), self.tr("Remove item"), self.press_remove_fav)
		self.button_menu.setMenu(self.fav_menu)
		#button_edit
		self.button_edit = QPushButton(QIcon(paths.edit_icon),"")
		self.button_edit.setStyleSheet(paths.BUTTONS_STYLE)
		self.button_edit.setMinimumHeight(30)
		self.button_edit.setMinimumWidth(30)
		self.button_edit.setCursor(Qt.PointingHandCursor)
		self.button_edit.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Preferred)
		self.button_edit.clicked.connect(self.press_edit_button)
		self.hbox_remove_edit.addWidget(self.button_edit)
		
		#autostart
		self.set_items()
########################################################################################

	def set_items(self):
		"""Set items"""
		self.table_view.setRowCount(0)
		paths.BASE_CURSOR.execute("SELECT * FROM favs")
		favs_list = paths.BASE_CURSOR.fetchall()
		for item in favs_list:
			QCoreApplication.processEvents()
			name = item[0]
			url = item[1]
			ch_id = item[2]
			self.add_one_row(name, url, ch_id)
			
	def add_one_row(self, name, url, *args):
		"""Add one row"""
		#name
		fileNameItem = MyTableItem()
		fileNameItem.setIcon(QIcon(paths.fav_icon))
		fileNameItem.setText(name)
		fileNameItem.URL = url
		fileNameItem.setFlags(fileNameItem.flags() ^ Qt.ItemIsEditable)
		fileNameItem.setTextAlignment(Qt.AlignVCenter | Qt.AlignLeft)
		if args:
			fileNameItem.CH_ID = args[0]
		#set
		row = self.table_view.rowCount()
		self.table_view.insertRow(row)
		self.table_view.setItem(row, 0, fileNameItem)
		
	def press_fav_item(self, item):
		"""Press fav item"""
		title = item.text()
		url = item.URL
		ch_id = item.CH_ID
		self.parent().parent().parent().press_any_item(title, url, ch_id)

	def press_remove_fav(self):
		"""Press remove Fav item"""
		selected_items = self.table_view.selectedItems()
		if len(selected_items) > 0:
			for current_item in selected_items:
				name = current_item.text()
				url = current_item.URL
				remove_params = [name, url]
				paths.BASE_CURSOR.execute("DELETE FROM favs WHERE zname=(?) AND zurl=(?)", remove_params)
				paths.BASE_CONNECTION.commit()
				self.table_view.removeRow(current_item.row())

	def press_edit_button(self):
		"""Press edit button"""
		current_item = self.table_view.currentItem()
		if current_item:
			name = current_item.text()
			url = current_item.URL
			self.edit_win = edit_window.Edit(name, url, current_item)
			self.edit_win.show()
			
	def press_add_button(self):
		"""Press add button"""
		self.win_add = add_station.Add_Station(self.parent().parent().parent())
		self.win_add.button_ok.clicked.connect(self.press_save)
		
	def press_save(self):
		"""Press save"""
		name = self.win_add.entry_name.text()
		url = self.win_add.entry_stream.text()
		if name and url:
			self.add_one_row(name, url)
			self.win_add.press_save()
			
	def press_save_favs(self):
		"""Press save all FAVS as playlist"""
		if self.table_view.rowCount() > 0:
			file_name = os.path.join(paths.path_playlists, 'playlist.m3u')
			filePath = QFileDialog.getSaveFileName(self, "Save playlist", file_name, ("Playlists (*.pls *.m3u *.m3u8)"), options = QFileDialog.DontUseNativeDialog)
			if filePath:
				path = filePath[0]
				if path:
					with open(path, 'a') as file_save:
						file_save.write('#EXTM3U\n')
						for row in range(self.table_view.rowCount()):
							item = self.table_view.item(row, 0)
							title = item.text()
							url = item.URL
							file_save.write('#EXTINF:-1,{0}\n'.format(title))
							file_save.write('{0}\n'.format(url))
			
#######################################################
class MyTableItem(QTableWidgetItem):
	def __init__(self):
		super().__init__()
		self.URL = None
		self.CH_ID = None

#######################################################
