import os
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from . import paths

class FilesTab(QWidget):
	def __init__(self):
		super().__init__()
		self.create_widgets()
		
	def create_widgets(self):
		self.CHOOSED_PATH = ""
		self.MAIN_PATH = QDir().homePath()
		
		self.vbox_main = QVBoxLayout()
		self.vbox_main.setContentsMargins(1,2,1,2)
		self.setLayout(self.vbox_main)
		
	#hbox_tools
		self.hbox_tools = QHBoxLayout()
		self.vbox_main.addLayout(self.hbox_tools)
		#combobox places
		self.combo_places = QComboBox()
		self.hbox_tools.addWidget(self.combo_places)
		#add drives
		self.add_drives()
		#
		list_pathes = [
					QDir().homePath(),
					os.path.join(QDir().homePath(), "Videos")
					]
		for item in list_pathes:
			self.combo_places.addItem(QIcon(paths.folder_icon), item)
		self.combo_places.addItem(QIcon(paths.folder_icon), paths.path_playlists)
		self.combo_places.setCurrentIndex(self.combo_places.findText(QDir.homePath()))
		self.combo_places.activated.connect(self.change_video_path)
		#button add folder
		self.add_folder_button = QPushButton()
		self.add_folder_button.setIcon(QIcon(paths.add_to_pls_icon))
		self.add_folder_button.setIconSize(QSize(25, 25))
		self.add_folder_button.setCursor(Qt.PointingHandCursor)
		self.add_folder_button.setStyleSheet(paths.BUTTONS_STYLE)
		self.add_folder_button.setMinimumHeight(30)
		self.hbox_tools.addWidget(self.add_folder_button)
		self.files_menu = QMenu()
		self.files_menu.addAction(QIcon(paths.add_file_icon), self.tr("Add selected file to PLS"), self.press_add_file_button)
		self.files_menu.addAction(QIcon(paths.folder_icon), self.tr("Add selected folder as playlist"), self.press_add_folder_button)
		self.files_menu.addAction(QIcon(paths.fav_icon), self.tr("Add selected file to Favs"), self.press_add_file_fav_button)
		self.add_folder_button.setMenu(self.files_menu)
		
		self.MAIN_PATH = self.combo_places.currentText()
		
	#create model
		self.model = QFileSystemModel()
		self.model.setReadOnly(True)
		self.model.setRootPath(self.MAIN_PATH)
		
		lists = ["*.mp4", "*.flv", "*.avi", "*.mpg", "*.mpeg", "*.mkv", "*.wmv", "*.m3u","*.m3u8","*.pls",]
		self.model.setNameFilters(lists)
		self.model.setNameFilterDisables(False)
		self.model.setFilter(QDir.Files | QDir.AllDirs | QDir.NoDotAndDotDot)
		#self.model.setFilter(QDir.AllDirs | QDir.NoDotAndDotDot)
		
		#treeview
		self.treeview = QTreeView()
		self.treeview.setMaximumWidth(300)
		self.treeview.setModel(self.model)
		self.treeview.hideColumn(1)
		self.treeview.hideColumn(2)
		self.treeview.hideColumn(3)
		self.treeview.setRootIndex(self.model.index(self.MAIN_PATH))
		self.treeview.header().hide()
		self.treeview.clicked.connect(self.press_folder_item)
		self.vbox_main.addWidget(self.treeview)
		
#######################################################################################
			
	def change_video_path(self):
		"""Change video path"""
		self.MAIN_PATH = self.combo_places.currentText()
		self.model.setRootPath(self.MAIN_PATH)
		self.treeview.setRootIndex(self.model.index(self.MAIN_PATH))
		
		
	def press_folder_item(self, index):
		"""Press folder item"""
		current_path = self.model.filePath(index)
		if QFileInfo(current_path).isDir():
			self.CHOOSED_PATH = current_path
		
	def press_add_folder_button(self):
		if self.CHOOSED_PATH:
			self.add_whole_folder(self.CHOOSED_PATH)
			self.parent().parent().parent().tabber.setCurrentIndex(1)
			
	def add_whole_folder(self, folder_path):
		"""Add whole folder"""
		self.parent().parent().parent().pls_tab.clear()
		for item in os.listdir(folder_path):
			file_path = os.path.join(folder_path, item)
			if QFileInfo(file_path).isFile() and QFileInfo(file_path).suffix().lower() in paths.VIDEO_LIST:
				name = QFileInfo(file_path).baseName()
				url = file_path
				self.parent().parent().parent().pls_tab.set_row(name, url)
			
	def press_add_file_button(self):
		"""Press add file to pls"""
		current_index = self.treeview.currentIndex()
		if current_index:
			current_path = self.treeview.model().filePath(current_index)
			self.add_one_file_to_pls(current_path)
			
	def add_one_file_to_pls(self, file_path):
		"""Add one file to pls"""
		title = os.path.split(file_path)[1]
		self.parent().parent().parent().pls_tab.set_row(title, file_path)
			
			
	def press_add_file_fav_button(self):
		"""Add file to FAV"""
		current_index = self.treeview.currentIndex()
		if current_index:
			url = self.treeview.model().filePath(current_index)
			name = os.path.split(url)[1]
			self.parent().parent().parent().add_to_fav(name, url)
			
	def add_drives(self):
		"""Add drives"""
		try:
			for item in QStorageInfo.mountedVolumes():
				if ('sda' in QTextCodec.codecForLocale().toUnicode(item.device()) and
					QFileInfo(item.displayName()).permission(QFileDevice.ReadUser)):
					drive_path = item.displayName()
					self.combo_places.addItem(QIcon(paths.hard_disc_icon), drive_path)
		except:
			self.combo_places.addItem(QIcon(paths.hard_disc_icon), QDir.rootPath())
