"""Kinokrad module"""
import re
import bs4
from bs4 import BeautifulSoup as BS
import urllib.parse
import urllib.request
from urllib.request import Request, urlopen
from PyQt5.QtWidgets import (QWidget, QVBoxLayout, QHBoxLayout, 
							 QLabel, QPushButton, QLineEdit, 
							 QComboBox, QScrollArea, QMenu,
							 QTableWidget, QTableWidgetItem, QAbstractItemView,
							 QHeaderView)
from PyQt5.QtGui import QPixmap, QFont, QIcon
from PyQt5.QtCore import Qt, QCoreApplication, QSize, QSettings, QUrl
from . import paths


class Kinokrad(QWidget):
	"""Kinoprofi class"""
	def __init__(self, parentWidget):
		super().__init__()
		self.parentWidget = parentWidget
		self.create_widgets()
		
	def create_widgets(self):
		"""Create widgets"""
		self.MAIN_URL = 'http://kinokrad.co/'
		self.PARSER = "html.parser"
		self.config_file = QSettings(paths.config_path, QSettings.IniFormat)
		
#vbox main
		self.vbox_main = QVBoxLayout()
		self.vbox_main.setContentsMargins(1, 1, 1, 1)
		self.setLayout(self.vbox_main)
		
		#line search
		self.hbox_search = QHBoxLayout()
		self.vbox_main.addLayout(self.hbox_search)
		#line
		self.line_search = QLineEdit()
		self.line_search.setPlaceholderText(self.tr("Search video"))
		self.line_search.setClearButtonEnabled(True)
		self.line_search.setMinimumHeight(paths.BUTTON_SIZE)
		self.line_search.returnPressed.connect(self.press_search)
		self.hbox_search.addWidget(self.line_search)
		#add button add station
		self.button_add_plsStation = QPushButton(QIcon(paths.add_to_pls_icon), "")
		icon = QIcon(paths.add_to_pls_icon)
		self.button_add_plsStation.setIconSize(QSize(25, 25))
		self.button_add_plsStation.setIcon(icon)
		self.button_add_plsStation.setMinimumHeight(paths.BUTTON_SIZE)
		self.button_add_plsStation.setMinimumWidth(paths.BUTTON_SIZE)
		self.button_add_plsStation.setStyleSheet(paths.BUTTONS_STYLE)
		self.button_add_plsStation.setCursor(Qt.PointingHandCursor)
		self.hbox_search.addWidget(self.button_add_plsStation)
		#menu
		self.add_menu = QMenu()
		self.add_menu.addAction(QIcon(paths.add_file_icon), self.tr("Add selected item to PLS"), self.press_add_item_button)
		self.add_menu.addAction(QIcon(paths.fav_icon), self.tr("Add selected item to Favs"), self.press_add_fav_button)
		self.button_add_plsStation.setMenu(self.add_menu)
		
	#table widget
		self.table_widget = QTableWidget(0, 1)
		self.vbox_main.addWidget(self.table_widget)
		self.table_widget.setShowGrid(True)
		self.table_widget.verticalHeader().hide()
		self.table_widget.horizontalHeader().hide()
		self.table_widget.setSelectionBehavior(QAbstractItemView.SelectRows)
		self.table_widget.setSelectionMode(QAbstractItemView.SingleSelection)
		self.table_widget.horizontalHeader().setSectionResizeMode(0, QHeaderView.Stretch)
		self.table_widget.itemDoubleClicked.connect(self.press_kp_item)
		
	#hbox_recent_close
		self.hbox_recent_close = QHBoxLayout()
		self.vbox_main.addLayout(self.hbox_recent_close)
		#button_load_recent
		self.button_load_recent = QPushButton()
		self.button_load_recent.setText(self.tr("Last video"))
		icon = QIcon("./icons/recents_icon.png")
		self.button_load_recent.setIconSize(QSize(25,25))
		self.button_load_recent.setIcon(icon)
		self.button_load_recent.setMinimumHeight(paths.BUTTON_SIZE)
		self.button_load_recent.setStyleSheet(paths.BUTTONS_STYLE)
		self.button_load_recent.setCursor(Qt.PointingHandCursor)
		self.button_load_recent.clicked.connect(self.press_load_recent)
		self.hbox_recent_close.addWidget(self.button_load_recent)
		#button_close_tab
		self.button_close_tab = QPushButton()
		self.button_close_tab.setText(self.tr("Close tab"))
		icon = QIcon("./icons/remove_icon2.png")
		self.button_close_tab.setIconSize(QSize(25,25))
		self.button_close_tab.setIcon(icon)
		self.button_close_tab.setMinimumHeight(paths.BUTTON_SIZE)
		self.button_close_tab.setStyleSheet(paths.BUTTONS_STYLE)
		self.button_close_tab.setCursor(Qt.PointingHandCursor)
		self.button_close_tab.clicked.connect(self.press_close_tab)
		self.hbox_recent_close.addWidget(self.button_close_tab)
		
		###set up
		self.setAttribute(Qt.WA_DeleteOnClose, True)
##################################################################################################

	def press_load_recent(self):
		"""Press load recent"""
		self.parse_items(self.MAIN_URL)

	def press_close_tab(self):
		"""Press close tab"""
		self.close()
		
	def get_post_params(self, text):
		"""Get post params"""
		#text = text.replace(' ', '+')
		#text = urllib.parse.quote(text)
		post_params = {
			'do' : "search",
			'story' : text,
			'subaction' : "search",
			'x' : 0,
			'y' : 0
		}
		return post_params

	def press_search(self):
		"""Press search"""
		self.table_widget.setRowCount(0)
		text = self.line_search.text().strip().lower()
		if text:
			post_params = self.get_post_params(text)
			post_args = urllib.parse.urlencode(post_params)
			binary_data = post_args.encode('utf-8')
			fp = urllib.request.urlopen(self.MAIN_URL, binary_data)
			soup = BS(fp, self.PARSER)
			all_divs = soup.find_all('div', attrs={'class':'postershort'})
			for item in all_divs:
				QCoreApplication.processEvents()
				title = item.a.img.get('title')
				url = item.a.get('href')
				self.add_one_row(title, url)
			
		
	def parse_items(self, url):
		"""Parse items"""
		req = Request(url, headers={'User-Agent': 'Mozilla/5.0'})
		webpage = urlopen(req).read()
		soup = BS(webpage, self.PARSER)
		all_divs = soup.find_all('div', attrs={'class':'postershort'})
		for item in all_divs:
			QCoreApplication.processEvents()
			title = item.a.img.get('title')
			url = item.a.get('href')
			self.add_one_row(title, url)
		
	def add_one_row(self, title, url):
		"""Add one row"""
		tableWidgetItem = MyTableItem()
		tableWidgetItem.setText(title)
		tableWidgetItem.setIcon(QIcon('./icons/kinokrad_icon.png'))
		tableWidgetItem.setFlags(tableWidgetItem.flags() ^ Qt.ItemIsEditable)
		tableWidgetItem.TITLE = title
		tableWidgetItem.setToolTip(title)
		tableWidgetItem.URL = url
		
		row = self.table_widget.rowCount()
		self.table_widget.insertRow(row)
		self.table_widget.setItem(row, 0, tableWidgetItem)
		
	def press_kp_item(self, tableItem):
		"""Press kp item"""
		title = tableItem.TITLE
		url = tableItem.URL
		video_url = self.get_video_url(url)
		self.parentWidget.player_play(video_url)
		self.parentWidget.set_name_channel(title)
		
	def get_video_url(self, url):
		"""Get video url"""
		req = Request(url, headers={'User-Agent': 'Mozilla/5.0'})
		webpage = urlopen(req).read()
		soup = BS(webpage, self.PARSER)
		video_tag = soup.find('div', attrs={'class':'boxfilm'})
		try:
			video_link = re.search(r'http.*m3u8', str(video_tag)).group(0)
		except:
			video_link = ''
		return video_link
			
	def press_add_item_button(self):
		"""Press add item to PLS"""
		current_item = self.table_widget.currentItem()
		title = current_item.TITLE
		url = current_item.URL
		video_url = self.get_video_url(url)
		self.parent().parent().parent().pls_tab.set_row(title, video_url)
		
	def press_add_fav_button(self):
		"""Press item to FAV"""
		current_item = self.table_widget.currentItem()
		if current_item:
			title = current_item.TITLE
			url = current_item.URL
			video_url = self.get_video_url(url)
			self.parent().parent().parent().open_add_win(title, video_url)
			
#####################################################################################

class MyTableItem(QTableWidgetItem):
	def __init__(self):
		super().__init__()
		self.TITLE = None
		self.URL = None
		
