"""Menu Bar module"""
from PyQt5.QtWidgets import QMenuBar, QMenu, QAction, QActionGroup
from PyQt5.QtGui import QIcon
from PyQt5.QtCore import Qt
from . import paths


class Menu_bar(QMenuBar):
	"""MenuBar class"""
	def __init__(self):
		super().__init__()
		self.create_menu()
		
	def create_menu(self):
		"""Create menu"""
	#menu_file
		self.menu_file = QMenu(self.tr("File"))
		self.addMenu(self.menu_file)
	#actions
		self.open_file_action = QAction(QIcon(paths.open_icon), self.tr("Open file(s)"), None)
		self.menu_file.addAction(self.open_file_action)
		self.open_folder_action = QAction(QIcon(paths.folder_icon), self.tr("Open directory"), None)
		self.menu_file.addAction(self.open_folder_action)
		self.open_pls_action = QAction(QIcon(paths.pls_icon), self.tr("Open PLS folder"), None)
		self.menu_file.addAction(self.open_pls_action)
		self.close_app_action = QAction(QIcon(paths.remove_icon), self.tr("Close"), None)
		self.menu_file.addAction(self.close_app_action)
	#playback
		self.menu_playback = QMenu(self.tr("Playback"))
		self.addMenu(self.menu_playback)
		self.play_action = QAction(QIcon(paths.play_icon), self.tr("Play"), None)
		self.menu_playback.addAction(self.play_action)
		self.pause_action = QAction(QIcon(paths.pause_icon), self.tr("Pause"), None)
		self.menu_playback.addAction(self.pause_action)
		self.stop_action = QAction(QIcon(paths.stop_icon), self.tr("Stop"), None)
		self.menu_playback.addAction(self.stop_action)
		self.next_file_action = QAction(QIcon("icons/next2.png"), self.tr("Next"), None)
		self.menu_playback.addAction(self.next_file_action)
		self.prev_file_action = QAction(QIcon("icons/prev.png"), self.tr("Prev"), None)
		self.menu_playback.addAction(self.prev_file_action)
		self.record_action = QAction(QIcon("icons/record.png"), self.tr("Record"), None)
		self.menu_playback.addAction(self.record_action)
	#video
		self.menu_video = QMenu(self.tr("Video"))
		self.addMenu(self.menu_video)
		#1
		self.fullscreen_action = QAction(QIcon(paths.fullscreen_icon), self.tr("Fullscreen"), None)
		self.menu_video.addAction(self.fullscreen_action)
		#2
		self.deinterlace_action = QAction(self.tr("Deinterlace"), None)
		self.deinterlace_action.setCheckable(True)
		self.menu_video.addAction(self.deinterlace_action)
		#3
		self.zoom_action = QAction(self.tr('Zoom'), None)
		self.zoom_action.setCheckable(True)
		self.menu_video.addAction(self.zoom_action)
		#4
		self.aspects_group = QActionGroup(self)
		self.aspects_group.setExclusive(True)
		self.sub_menu_aspects = QMenu(self.tr('Aspects'))
		self.menu_video.addMenu(self.sub_menu_aspects)
		self.action_no_aspect = QAction(self.tr('Default'), None)
		self.action_aspect_4_3 = QAction('4:3', None)
		self.action_aspect_16_9 = QAction('16:9', None)
		self.action_no_aspect.setCheckable(True)
		self.action_aspect_4_3.setCheckable(True)
		self.action_aspect_16_9.setCheckable(True)
		self.action_no_aspect.setChecked(True)
		self.sub_menu_aspects.addAction(self.action_no_aspect)
		self.sub_menu_aspects.addAction(self.action_aspect_4_3)
		self.sub_menu_aspects.addAction(self.action_aspect_16_9)
		self.aspects_group.addAction(self.action_no_aspect)
		self.aspects_group.addAction(self.action_aspect_4_3)
		self.aspects_group.addAction(self.action_aspect_16_9)
		#5
		self.screenshot_action = QAction(QIcon('./icons/screenshot_icon.gif'), self.tr("Take screenshot"), None)
		self.menu_video.addAction(self.screenshot_action)
	#subtitles
		self.menu_subs = QMenu(self.tr('Subtitles'))
		self.addMenu(self.menu_subs)
		#1
		self.open_subtitles_action = QAction(QIcon('./icons/subtitles_icon.gif'), self.tr('Open subtitles'), None)
		self.menu_subs.addAction(self.open_subtitles_action)
	#tools
		self.menu_tools = QMenu(self.tr("Tools"))
		self.addMenu(self.menu_tools)
		self.add_stream_action = QAction(QIcon(paths.add_channel_icon), self.tr("Add stream"), None)
		self.menu_tools.addAction(self.add_stream_action)
		self.create_pls_action = QAction(QIcon(paths.create_pls_icon), self.tr("Create playlist"), None)
		self.menu_tools.addAction(self.create_pls_action)
		self.tvgid_action = QAction(QIcon(paths.tvgid_icon), self.tr("TVGid"), None)
		self.menu_tools.addAction(self.tvgid_action)
		self.backup_action = QAction(QIcon("./icons/zip_icon.png"), self.tr("Backup"), None)
		self.menu_tools.addAction(self.backup_action)
	##ad_tabs
		self.menu_add_tabs = QMenu(self.tr('Add Tabs'))
		self.menu_add_tabs.setIcon(QIcon('./icons/add_tab.png'))
		self.kinokrad_action = QAction(QIcon('./icons/kinokrad_icon.png'), self.tr("Kinokrad"), None)
		self.menu_add_tabs.addAction(self.kinokrad_action)
		self.menu_tools.addMenu(self.menu_add_tabs)
	#about
		self.menu_about = QMenu(self.tr("About"))
		self.addMenu(self.menu_about)
		self.guide_action = QAction(QIcon(paths.howto_icon), self.tr("Guide"), None)
		self.menu_about.addAction(self.guide_action)
		self.video_review_action = QAction(QIcon(paths.yt_icon), self.tr("Video review"), None)
		self.menu_about.addAction(self.video_review_action)
		self.about_action = QAction(QIcon(paths.about_icon), self.tr("About"), None)
		self.menu_about.addAction(self.about_action)
		
