import os
import sys
import sqlite3
import shutil
from PyQt5.QtCore import QSettings, QCoreApplication, QStandardPaths


#paths
if sys.platform.startswith('win'):
	#paths
	SYS_CONFIG_DIR = QStandardPaths.standardLocations(QStandardPaths.AppDataLocation)
	path_program = os.path.join(SYS_CONFIG_DIR, "ZVVOnlineTV")
	path_playlists = os.path.join(path_program, "playlists")
	config_path = os.path.join(path_program, "config.ini")
	records_path = os.path.join(path_program, "records")
	bookmarks_path = os.path.join(path_program, "bookmarks")
	base_path = os.path.join(path_program, "zvvtvbase.db")
	channels_base = os.path.join(os.getcwd(), "zvvchannels.db")
	
if sys.platform.startswith('linux'):
	#paths
	SYS_CONFIG_DIR = os.path.join(os.getenv("HOME"), ".config")
	path_program = os.path.join(SYS_CONFIG_DIR, "ZVVOnlineTV")
	path_playlists = os.path.join(path_program, "playlists")
	config_path = os.path.join(path_program, "config.ini")
	records_path = os.path.join(path_program, "records")
	bookmarks_path = os.path.join(path_program, "bookmarks")
	base_path = os.path.join(path_program, "zvvtvbase.db")
	channels_base = os.path.join(os.getcwd(), "zvvchannels.db")

if not os.path.exists(path_program):
	os.mkdir(path_program)
	
if not os.path.exists(path_playlists):
	os.mkdir(path_playlists)
		
	list_pls = os.path.join(os.getcwd(), "playlists")
	pls_array = os.listdir(list_pls)
	for item in pls_array:
		src_path = os.path.join(list_pls, item)
		shutil.copy(src_path, path_playlists)

if not os.path.exists(config_path) or not QSettings(config_path, QSettings.IniFormat).contains("config/version") or int(QSettings(config_path, QSettings.IniFormat).value("config/version")) < 14:
	src_path = os.path.join(os.getcwd(), "config.ini")
	shutil.copy(src_path, config_path)
	if int(QSettings(config_path, QSettings.IniFormat).value("config/version")) < 11:
		src = os.path.join(os.getcwd(), "zvvtvbase.db")
		shutil.copy(src, base_path)
	
if not os.path.exists(bookmarks_path):
	os.mkdir(bookmarks_path)
			
if not os.path.exists(base_path):
	src = os.path.join(os.getcwd(), "zvvtvbase.db")
	shutil.copy(src, base_path)
	
	
BASE_CONNECTION = sqlite3.connect(base_path)
BASE_CURSOR = BASE_CONNECTION.cursor()

CH_CONNECTION = sqlite3.connect(channels_base)
CH_CURSOR = CH_CONNECTION.cursor()

TV_ICONS_PATH = os.path.join(os.getcwd(), "tvgid/tvicons")

#set ups
BUTTON_SIZE=30

#STYLE

BUTTONS_STYLE = ("""QPushButton{
						border:1px solid silver;
						border-radius:6px;
					} 
					QPushButton::hover{
						border:1px solid silver; 
						background-color:#fff19b;
					} 
					QPushButton::checked{
						border: 1px solid silver; 
						background-color:lightgreen;
					}""")

#icons
icons_folder_path = "icons"

fav_icon = "icons/fav_icon.png"
url_icon = "icons/url.png"
pls_icon = "icons/playlist.png"
sub_pls_icon = os.path.join(icons_folder_path, "sub_pls_icon.png")
add_pls_icon = "icons/add_pls_icon.png"
add_folder_icon = "icons/add_folder_icon.png"
add_file_icon = "icons/add_file_icon.png"
video_icon = "icons/video_icon.png"
serial_icon = os.path.join(icons_folder_path, "serial_icon2.png")
play_icon = "icons/blue_play.png"
stop_icon = "icons/stop_mode.png"
pause_icon = "icons/yellow_pause2.png"
stop_mode = "icons/stop_mode.png"
replay_mode = "icons/replay_mode2.png"
next_mode = "icons/next_mode.png"
shuffle_mode = "icons/shuffle_mode.png"
mini_icon = "icons/minimize_icon2.png"
folder_icon = "icons/folder2.png"
menu_icon = os.path.join(icons_folder_path, "menu_icon.png")
open_icon = os.path.join(icons_folder_path, "open_icon.png")
load_icon = os.path.join(icons_folder_path, "load_icon.png")
about_icon = "icons/about_icon2.png"
hard_disc_icon = "icons/hard_disk.png"
trash_icon = "icons/trash.png"
save_icon = "icons/save_icon.png"
save_icon_local = os.path.join(icons_folder_path, "save_icon_local.png")
save_icon_web = os.path.join(icons_folder_path, "save_icon_web.png")
lock_icon = "icons/lock_icon.png"
remove_icon = "icons/remove_icon2.png"
remove_pls_icon = "icons/remove_pls.png"
remove_hand = os.path.join(icons_folder_path, "hand_remove_icon.png")
add_to_pls_icon = "icons/add_to_pls.png"
create_pls_icon = os.path.join(icons_folder_path, "create_pls_icon.png")
yt_icon = "icons/YT.png"
config_icon = "icons/config_icon.png"
fullscreen_icon = "icons/full_blue.png"
volume_icon = "icons/volume_icon.png"
volume_mute_icon = "icons/volume_mute_icon.png"
search_icon = os.path.join(icons_folder_path, "search_icon.png")
add_channel_icon = os.path.join(icons_folder_path, "add_ch.png")
options_icon = os.path.join(icons_folder_path, "options_icon.png")
arrow_up_icon = os.path.join(icons_folder_path, "arrow_up_icon.png")
arrow_down_icon = os.path.join(icons_folder_path, "arrow_down_icon.png")
replay_pls_icon = os.path.join(icons_folder_path, "replay_pls.png")
stop_pls_icon = os.path.join(icons_folder_path, "stop_pls.png")
sleep_icon = os.path.join(icons_folder_path, "sleep_icon.png")
tvgid_icon = os.path.join(icons_folder_path, "tv/tvgid_icon.png")
ukraine_flag = os.path.join(icons_folder_path, "Ukraine.png")
russia_flag = os.path.join(icons_folder_path, "Russia.png")
edit_icon = os.path.join(icons_folder_path, "edit_button.png")
info_icon = os.path.join(icons_folder_path, "info_icon.png")
sub_item_icon = os.path.join(icons_folder_path, "sub_item_icon4.png")
howto_icon = os.path.join(icons_folder_path, "howto_icon.png")

ua_icon = os.path.join(icons_folder_path, "Ukraine.png")
bookmarks_icon = os.path.join(icons_folder_path, "bookmark.png")
video_cassette_icon = os.path.join(icons_folder_path, "retro_icon.png")
retro_icon = os.path.join(icons_folder_path, "retro_cam_icon.png")

#files
videos_reg_exp = "(*.mp4 *.flv *.avi *.mkv *.wmv *.mpeg *.mpg *.vob *.webm)"
VIDEO_LIST = ["mpg","flv","mp4","mpeg","avi","mkv","vob","webm","wmv",]
PLS_LIST = ["m3u","m3u8","pls",]

search_tab_url = "https://anwap.film/films/search"
search_serials_url = "http://anwap.org/serials/search"
search_tab_start_url = "http://anwap.film"

howto_index = os.path.join(os.getcwd(), "howto/howto.html")
#######################################################################################################

def get_scrollbar_style():
	"""Get scrollbar style"""
	SCROLL_BAR_STYLE = ("""QScrollBar:vertical {           
				border: 1px solid #999999;
				background:white;
				width:10px;
				margin: 0px 0px 0px 0px;
			}
			QScrollBar::handle:vertical {
				background:lightblue;
				min-height: 0px;
			}
			QScrollBar::add-line:vertical {
				background: lightblue;
				height: 0px;
				subcontrol-position: bottom;
				subcontrol-origin: margin;
				}
			QScrollBar::sub-line:vertical {
				background:lightblue;
				height: 0 px;
				subcontrol-position: top;
				subcontrol-origin: margin;
				}
			""")
	return SCROLL_BAR_STYLE

