"""PLS module"""
import os
import re
import urllib
import urllib.request
from zipfile import ZipFile as ZP
import mimetypes as MT
from PyQt5.QtWidgets import (QWidget, QVBoxLayout, QHBoxLayout, 
							 QLabel, QLineEdit, QPushButton, 
							 QComboBox, QTableWidget, QTableWidgetItem,
							 QAbstractItemView, QHeaderView, QFrame,
							 QMenu, QFileDialog, QTableView, QTreeView,
							 QAbstractItemView)
from PyQt5.QtGui import (QIcon, QFont, QKeySequence, QStandardItemModel,
							 QStandardItem, QDesktopServices)
from PyQt5.QtCore import (Qt, QCoreApplication, QSize, QDir,
						  QFileInfo, QSortFilterProxyModel, QUrl,
						  QModelIndex)
from . import paths, new_pls, zvvfsmodel

class Pls(QWidget):
	def __init__(self):
		super().__init__()
		self.SUB_ITEMS_DICT = {}
		self.PATH_PLS = paths.path_playlists
		self.create_widgets()

	def create_widgets(self):
		"""Create widgets"""
		#init
		self.mimetype = MT.MimeTypes()
#set vbox_main
		self.vbox_main = QVBoxLayout()
		self.vbox_main.setContentsMargins(3, 3, 3, 3)
		self.setLayout(self.vbox_main)
		
	#hbox_tools
		self.hbox_tools = QHBoxLayout()
		self.vbox_main.addLayout(self.hbox_tools)
		#button_open_pls
		self.button_open_file = Button_tools()
		self.button_open_file.setIcon(QIcon(paths.open_icon))
		self.button_open_file.clicked.connect(self.press_open_file)
		self.hbox_tools.addWidget(self.button_open_file)
		#button add to fav
		self.button_add_plsStation = Button_tools()
		self.button_add_plsStation.setIcon(QIcon(paths.fav_icon))
		self.button_add_plsStation.setToolTip(self.tr("Add to Favs"))
		self.button_add_plsStation.clicked.connect(self.press_add_plsStation)
		self.hbox_tools.addWidget(self.button_add_plsStation)
		#button search
		self.search_button = Button_tools()
		self.search_button.setIcon(QIcon(paths.search_icon))
		self.search_button.clicked.connect(self.press_search_button)
		self.hbox_tools.addWidget(self.search_button)
		#button_pls_options
		self.button_pls_options = Button_tools()
		self.button_pls_options.setIcon(QIcon(paths.options_icon))
		self.button_pls_options.clicked.connect(self.press_button_pls_options)
		self.hbox_tools.addWidget(self.button_pls_options)
		#
		self.hbox_tools.addStretch()
		
	#add hbox open playlist
		self.hbox_open_pls = QHBoxLayout()
		self.vbox_main.addLayout(self.hbox_open_pls)
		
		#filesystemmodel
		self.model = zvvfsmodel.MyFileSystemModel()
		self.model.setReadOnly(True)
		self.model.setRootPath(self.PATH_PLS)
		
		#add combo playlists
		self.combo_playlist = QComboBox()
		self.combo_playlist.setModel(self.model)
		self.combo_playlist.setRootModelIndex(self.model.index(self.PATH_PLS))
		self.combo_playlist.setMinimumHeight(paths.BUTTON_SIZE)
		self.combo_playlist.activated.connect(self.choose_pls)
		self.hbox_open_pls.addWidget(self.combo_playlist)

	#combo_folder_pls
		self.combo_folder_pls = QComboBox()
		self.combo_folder_pls.hide()
		self.combo_folder_pls.activated.connect(self.press_choose_folder_pls)
		self.vbox_main.addWidget(self.combo_folder_pls)
		
	#add combo categories
		self.combo_sub_pls = QComboBox()
		self.combo_sub_pls.hide()
		self.combo_sub_pls.activated.connect(self.press_choose_sub_pls)
		self.vbox_main.addWidget(self.combo_sub_pls)

	#hbox pls url
		self.hbox_pls_url = QHBoxLayout()
		self.vbox_main.addLayout(self.hbox_pls_url)
		#.pls line
		self.pls_line = QLineEdit()
		self.pls_line.setPlaceholderText(self.tr("put playlist url here"))
		self.pls_line.setMinimumHeight(paths.BUTTON_SIZE)
		self.pls_line.setClearButtonEnabled(True)
		self.pls_line.returnPressed.connect(self.press_pls_line)
		self.hbox_pls_url.addWidget(self.pls_line)
		#..load button
		self.pls_load_url_button = Button_tools()
		self.pls_load_url_button.setIcon(QIcon(paths.load_icon))
		self.pls_load_url_button.clicked.connect(self.press_pls_line)
		self.hbox_pls_url.addWidget(self.pls_load_url_button)

	#hbox search
		self.hbox_search = QHBoxLayout()
		self.vbox_main.addLayout(self.hbox_search)
		#.search line
		self.search_line = QLineEdit()
		self.search_line.setPlaceholderText(self.tr("Search in list below"))
		self.search_line.setClearButtonEnabled(True)
		self.search_line.textChanged.connect(self.press_search)
		self.search_line.hide()
		self.hbox_search.addWidget(self.search_line)
		
	#TABLE_VIEW
		#model
		self.model = QStandardItemModel()
		self.model.setHorizontalHeaderLabels(("Name",))
		
		#sort_model
		self.sort_model = QSortFilterProxyModel()
		self.sort_model.setSourceModel(self.model)
		#self.sort_model.setFilterKeyColumn(1)
		self.sort_model.setFilterCaseSensitivity(Qt.CaseInsensitive)

	#table view
		self.table_view = MyTableView()
		self.table_view.setModel(self.sort_model)
		self.table_view.setSelectionMode(QAbstractItemView.SingleSelection)
		self.table_view.verticalHeader().hide()
		self.table_view.horizontalHeader().hide()
		self.table_view.setShowGrid(True)
		self.table_view.doubleClicked.connect(self.press_pls_item)
		self.vbox_main.addWidget(self.table_view)
		self.table_view.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
		
	#vbox pls_options
		self.widget_pls_options = QFrame()
		self.widget_pls_options.setFrameShape(QFrame.StyledPanel)
		self.widget_pls_options.hide()
		self.vbox_main.addWidget(self.widget_pls_options)
		self.vbox_pls_options = QVBoxLayout()
		self.vbox_pls_options.setContentsMargins(2, 2, 2, 2)
		self.widget_pls_options.setLayout(self.vbox_pls_options)
		#move item
		self.hbox_move_item = QHBoxLayout()
		self.vbox_pls_options.addLayout(self.hbox_move_item)
		#button-Up
		self.button_up = QPushButton()
		self.button_up.setIcon(QIcon(paths.arrow_up_icon))
		self.button_up.setCursor(Qt.PointingHandCursor)
		self.button_up.setStyleSheet("QPushButton::hover{background-color:lightgreen;}")
		self.button_up.clicked.connect(self.press_pls_button_move_up)
		self.hbox_move_item.addWidget(self.button_up)
		#button-Down
		self.button_down = QPushButton()
		self.button_down.setIcon(QIcon(paths.arrow_down_icon))
		self.button_down.setCursor(Qt.PointingHandCursor)
		self.button_down.setStyleSheet("QPushButton::hover{background-color:#fcff91;}")
		self.button_down.clicked.connect(self.press_pls_button_move_down)
		self.hbox_move_item.addWidget(self.button_down)
		
	#save_remove hbox
		self.save_remove_hbox = QHBoxLayout()
		self.vbox_main.addLayout(self.save_remove_hbox)
		#save button
		self.menu_button = QPushButton(QIcon(paths.menu_icon), self.tr("PLS Menu"))
		self.menu_button.setStyleSheet(paths.BUTTONS_STYLE)
		self.menu_button.setFixedHeight(30)
		self.menu_button.setCursor(Qt.PointingHandCursor)
		self.save_remove_hbox.addWidget(self.menu_button)
		#save_menu
		font = QFont()
		font.setPointSize(12)
		self.pls_menu = QMenu()
		self.pls_menu.setFont(font)
		self.pls_menu.addAction(QIcon(paths.folder_icon), self.tr("Open PLS folder"), self.press_open_manual, QKeySequence("Ctrl+O"))
		self.pls_menu.addAction(QIcon(paths.create_pls_icon), self.tr("Create/edit PLS"), self.press_create_pls, QKeySequence("Ctrl+E"))
		self.pls_menu.addSeparator()
		self.pls_menu.addAction(QIcon(paths.save_icon), self.tr("Save as local PLS"), self.press_save_local)
		self.pls_menu.addAction(QIcon(paths.url_icon), self.tr("Save as Url"), self.press_save_web)
		self.pls_menu.addSeparator()
		self.pls_menu.addAction(QIcon(paths.remove_icon), self.tr("Remove one item"), 
								self.press_remove_one_item)
		self.pls_menu.addAction(QIcon('./icons/clear_pls_icon.png'), self.tr("Clear"), 
								self.clear)
		self.menu_button.setMenu(self.pls_menu)
		
#################################################################################################

	def get_normal_item(self, modelIndex):
		"""Get normal index"""
		proxy = modelIndex.model()
		proxyIndex = proxy.index(modelIndex.row(), 0)
		if proxy == QStandardItemModel():
			standard_item = self.model.itemFromIndex(proxyIndex)
		else:
			sourceIndex = proxy.mapToSource(proxyIndex)
			standard_item = self.model.itemFromIndex(sourceIndex)
		return standard_item

	def press_add_plsStation(self):
		"""Press add item to FAV"""
		current_index = self.table_view.currentIndex()
		if current_index.row() >= 0:
			standard_item = self.get_normal_item(current_index)
			name = standard_item.TITLE
			url = standard_item.URL
			self.parent().parent().parent().add_to_fav(name, url)

	def press_pls_button_move_up(self):
		"""Move pls item up"""
		current_index = self.table_view.currentIndex()
		if current_index and current_index.row() > 0:
			current_item = self.get_normal_item(current_index)
			above_index = self.sort_model.index((current_index.row()-1), 0)
			above_item = self.get_normal_item(above_index)
			above_name = above_item.TITLE
			above_url = above_item.URL
			#current name
			current_name = current_item.TITLE
			current_url = current_item.URL
			#above item
			above_item.setText(current_name)
			above_item.TITLE = current_name
			above_item.URL = current_url
			#move
			current_item.setText(above_name)
			current_item.TITLE = above_name
			current_item.URL = above_url
			self.table_view.setCurrentIndex(above_index)
			
	def press_pls_button_move_down(self):
		"""Press move item down"""
		current_index = self.table_view.currentIndex()
		if current_index and current_index.row() < (self.model.rowCount() - 1):
			current_item = self.get_normal_item(current_index)
			bottom_index = self.sort_model.index((current_index.row()+1), 0)
			bottom_item = self.get_normal_item(bottom_index)
			bottom_name = bottom_item.TITLE
			bottom_url = bottom_item.URL
			#current name
			current_name = current_item.TITLE
			current_url = current_item.URL
			#bottom item
			bottom_item.setText(current_name)
			bottom_item.TITLE = current_name
			bottom_item.URL = current_url
			#move
			current_item.setText(bottom_name)
			current_item.TITLE = bottom_name
			current_item.URL = bottom_url
			self.table_view.setCurrentIndex(bottom_index)
			
	def press_choose_sub_pls(self, text):
		"""Press choose sub pls"""
		current_key = self.combo_sub_pls.currentText()
		self.clear()
		if len(self.SUB_ITEMS_DICT.keys()) > 0:
			list_items = self.SUB_ITEMS_DICT[current_key]
			for item in list_items:
				QCoreApplication.processEvents()
				name = item[0][0]
				url = item[0][1]
				self.set_row(name, url)
		
	def press_search_button(self):
		"""Press search button"""
		if self.search_line.isVisible():
			self.search_line.hide()
		else:
			self.search_line.show()
			self.search_line.setFocus()
			
	def press_button_pls_options(self):
		"""Press pls options"""
		if self.widget_pls_options.isVisible():
			self.widget_pls_options.hide()
		else:
			self.widget_pls_options.show()
			
	def press_search(self, text):
		"""Press search"""
		self.sort_model.setFilterRegExp(text)
		
	def press_open_file(self):
		"""Press open file"""
		fileName = QFileDialog.getOpenFileName(self, self.tr("Open File"), 
						QDir().homePath(), self.tr("Files (*.pls *.m3u *.m3u8)"))
		if fileName[0]:
			fileName = fileName[0]
			if len(fileName) > 0:
				self.clear()
				self.set_pls(fileName)
			
	def choose_pls(self):
		"""Choose playlist"""
		self.combo_sub_pls.clear()
		self.combo_sub_pls.hide()
		self.clear()
		self.combo_folder_pls.hide()
		self.combo_folder_pls.clear()
		current_pls = self.combo_playlist.currentText()
		full_path_pls = os.path.join(self.PATH_PLS, current_pls)
		self.checking_pls(full_path_pls)
			
	def checking_pls(self, path):
		"""Check pls type"""
		self.clear()
		if os.path.isdir(path):
			self.combo_folder_pls.clear()
			if len(os.listdir(path)) > 1:
				list_folders_item = os.listdir(path)
				list_folders_item.sort()
				for item in list_folders_item:
					if os.path.isfile(os.path.join(path, item)):
						if not QFileInfo(os.path.join(path, item)).isHidden():
							if "(URL)" in item:
								self.combo_folder_pls.addItem(QIcon(paths.url_icon), item)
							else:
								self.combo_folder_pls.addItem(QIcon(paths.sub_pls_icon), item)
				self.combo_folder_pls.show()
			else:
				self.combo_folder_pls.hide()
		else:
			try:
				if "zip" in self.mimetype.guess_type(path)[0]:
					self.combo_folder_pls.clear()
					list_zip_files = ZP(path).namelist()
					for item in list_zip_files:
						if QFileInfo(item).suffix().lower() in paths.PLS_LIST:
							self.combo_folder_pls.addItem(QIcon("icons/sub_pls_icon.png"), item)
							self.combo_folder_pls.show()
				else:
					self.combo_folder_pls.hide()
					self.set_pls_items(path)
			except:
				self.combo_folder_pls.hide()
				self.set_pls_items(path)
	
	def set_pls_items(self, path):
		if "(URL)".lower() in path.lower():
			file_open = open(path, "r")
			data = file_open.read()
			file_open.close()
			self.pls_line.setText(data)
			self.press_pls_line()
		else:
			self.set_pls(path)
			
	def press_choose_folder_pls(self):
		"""Press choose folder pls"""
		self.combo_sub_pls.clear()
		self.clear()
		folder_path = os.path.join(self.PATH_PLS, self.combo_playlist.currentText())
		playlist_path = os.path.join(folder_path, self.combo_folder_pls.currentText())
		self.set_pls_items(playlist_path)

	def press_save_local(self):
		"""Save local pls"""
		if self.model.rowCount() > 0:
			file_name = os.path.join(paths.path_playlists, 'playlist.m3u')
			filePath = QFileDialog.getSaveFileName(self, "Save playlist", file_name, ("Playlists (*.pls *.m3u *.m3u8)"), options = QFileDialog.DontUseNativeDialog)
			if filePath:
				path = filePath[0]
				if path:
					with open(path, 'a') as file_save:
						file_save.write('#EXTM3U\n')
						for i in range(0, (self.model.rowCount())):
							tableItem = self.get_normal_item(self.sort_model.index(i, 0))
							title = tableItem.TITLE
							url = tableItem.URL
							file_save.write('#EXTINF:-1,{0}\n'.format(title))
							file_save.write('{0}\n'.format(url))
			
	def press_save_web(self):
		"""Press save web"""
		from . import create_pls
		text = self.pls_line.text()
		if text:
			self.create_pls_win = create_pls.Create_pls()
			self.create_pls_win.SAVE_URL = text
			self.create_pls_win.show()
	
	def press_pls_line(self):
		"""Press pls line"""
		self.clear()
		if self.combo_folder_pls.isVisible():
			current_path = os.path.join(self.PATH_PLS, self.combo_playlist.currentText())
			if os.path.isfile(current_path):
				self.combo_folder_pls.hide()
		pls_url = self.pls_line.text()
		if pls_url:
			try:
				content = urllib.request.urlopen(pls_url)
				self.open_playlist(content, "web")
			except:
				self.pls_line.clear()

	def set_pls(self, path_pls):
		"""Set pls"""
		if os.path.isfile(os.path.split(path_pls)[0]):
			if "zip" in self.mimetype.guess_type(os.path.split(path_pls)[0])[0]:
				zip_path = os.path.split(path_pls)[0]
				zip_file = os.path.split(path_pls)[1]
				data = ZP(zip_path).open(zip_file).readlines()
				self.open_playlist(data, "web")
		else:
			try:
				file_open = open(path_pls, 'r', encoding='utf-8')
				data = file_open.readlines()
			except:
				file_open = open(path_pls, 'r', encoding='cp1251')
				data = file_open.readlines()
			file_open.close()
			self.open_playlist(data, "local")
		
	def open_playlist(self, content, type_file):
		"""Open playlist"""
		self.SUB_ITEMS_DICT.clear()
		QCoreApplication.processEvents()
		if self.combo_sub_pls.isVisible():
			self.combo_sub_pls.hide()
		if self.combo_sub_pls.count() > 0:
			self.combo_sub_pls.clear()
		cat_name = None
		cat_item = []
		name = re.compile(r"#EXTINF:.*", re.I)
		url = re.compile(r"http://|https://.*|rtsp://|udp://|rtp://", re.I)
		name_station = ''
		url_station = ''
		for line in content:
			#QCoreApplication.processEvents()
			if type_file == "web":
				line = line.decode()
			line_file = line.split("\n")[0]
			#add cat
			if "group-title" in line_file or "#EXTGRP" in line_file:
				#self.combo_sub_pls.show()
				if "group-title" in line_file:
					cat_name = re.search(r'group-title=".*"', line).group(0).split("\"")[1].strip()
				else:
					cat_name = line_file.split(":")[1].strip()
				if not self.SUB_ITEMS_DICT.get(cat_name):
					self.SUB_ITEMS_DICT[cat_name] = []
			if name.search(line_file):
				if "," in line_file:
					line_file = str(line_file)
					name_station = line_file.split(',')[-1]
					name_station = name_station.strip()
			if '://' in line_file and line_file[0] != "#":
				url_station = line_file.strip()
			else:
				if os.path.isfile(line_file) and os.path.exists(line_file):
					url_station = line_file
			if url_station:
				if not name_station:
					name_station = "station"
			if name_station and url_station:
				if cat_name:
					cat_item.append((name_station, url_station))
				self.set_row(name_station, url_station)
				name_station = ''
				url_station = ''
				if cat_item:
					self.SUB_ITEMS_DICT[cat_name].append(cat_item)
					cat_item = []
		if len(self.SUB_ITEMS_DICT.keys()) > 0:
			for item in self.SUB_ITEMS_DICT.keys():
				self.combo_sub_pls.addItem(QIcon(paths.sub_item_icon), item)
			self.combo_sub_pls.show()
		else:
			self.combo_sub_pls.clear()
			if self.combo_sub_pls.isVisible():
				self.combo_sub_pls.hide()
		
	def set_row(self, name, url, *args):
		"""Create one pls row"""
		#name
		fileNameItem = MyItem()
		fileNameItem.setText(name)
		fileNameItem.setToolTip(name)
		fileNameItem.TITLE = name
		fileNameItem.URL = url
		#set
		self.model.appendRow([fileNameItem,])
		
	def press_pls_item(self, modelIndex):
		"""Press PLS item"""
		standard_item = self.get_normal_item(modelIndex)
		title = standard_item.TITLE
		url = standard_item.URL
		self.parent().parent().parent().press_any_item(title, url)

	def clear(self):
		"""Clear pls list"""
		self.model.clear()
		
	def press_remove_button(self):
		if self.combo_playlist.currentIndex() > 0:
			current_pls = self.combo_playlist.currentText()
			full_pls_path = os.path.join(self.PATH_PLS, current_pls)
			if os.path.exists(full_pls_path):
				if os.path.isdir(full_pls_path):
					if self.combo_folder_pls.isVisible() and self.combo_folder_pls.count() > 0:
						folder_pls_item = self.combo_folder_pls.currentText()
						full_pls_path = os.path.join(full_pls_path, folder_pls_item)
						if os.path.isfile(full_pls_path):
							os.remove(full_pls_path)
							self.combo_folder_pls.removeItem(self.combo_folder_pls.findText(folder_pls_item))
							self.combo_folder_pls.setCurrentIndex(0)
				elif 'zip' in self.mimetype.guess_type(full_pls_path)[0]:
					if os.path.isfile(full_pls_path):
						os.remove(full_pls_path)
						self.combo_playlist.removeItem(self.combo_playlist.findText(current_pls))
						self.combo_playlist.setCurrentIndex(0)
						if self.combo_folder_pls.isVisible():
							self.combo_folder_pls.clear()
							self.combo_folder_pls.hide()
				else:
					if os.path.isfile(full_pls_path):
						os.remove(full_pls_path)
						self.clear()
						self.combo_playlist.removeItem(self.combo_playlist.findText(current_pls))
						self.combo_playlist.setCurrentIndex(0)
			
	def press_remove_one_item(self):
		"""Remove one item from pls"""
		current_index = self.table_view.currentIndex()
		if current_index:
			current_item_row = current_index.row()
			self.sort_model.removeRow(current_item_row)
			
	def press_open_manual(self):
		"""Open PLS folder in system file explorer"""
		QDesktopServices.openUrl(QUrl(paths.path_playlists))

	def press_create_pls(self):
		"""Show create pls win"""
		self.create_win = new_pls.New_pls()
		
###################################################################################################

class MyTableView(QTableView):
	"""MyTableView class"""
	def __init__(self):
		super().__init__()
		self.setAcceptDrops(True)
	
	def dragEnterEvent(self, event):
		event.accept()
		
	def dragMoveEvent(self, event):
		event.accept()
		
	def dropEvent(self, e):
		list_files = e.mimeData().urls()
		for item in list_files:
			path = QUrl(item.url()).toLocalFile()
			if os.path.exists(path):
				if os.path.isfile(path):
					if QFileInfo(path).suffix().lower() in paths.VIDEO_LIST:
						title = os.path.split(path)[1]
						self.parent().set_row(title, path)
					if QFileInfo(path).suffix() in paths.PLS_LIST:
						self.parent().checking_pls(path)
		
###################################################################################################

class MyItem(QStandardItem):
	"""My TableWidgetItem class"""
	def __init__(self):
		super().__init__()
		self.TITLE = None
		self.URL = None
		self.setFlags(self.flags() ^ Qt.ItemIsEditable)
		self.setTextAlignment(Qt.AlignVCenter | Qt.AlignLeft)
		
####################################################################################################

class Button_tools(QPushButton):
	"""Button tools class"""
	def __init__(self):
		super().__init__()
		self.create_button()
		
	def create_button(self):
		"""Create button"""
		self.setIconSize(QSize(25, 25))
		self.setFixedSize(paths.BUTTON_SIZE, paths.BUTTON_SIZE)
		self.setStyleSheet(paths.BUTTONS_STYLE)
		self.setCursor(Qt.PointingHandCursor)
		
