"""Search module"""
import os
import json
import re
import bs4
from bs4 import BeautifulSoup as BS
import urllib.parse
import urllib.request
from urllib.request import Request, urlopen, FancyURLopener
from PyQt5.QtWidgets import (QWidget, QVBoxLayout, QHBoxLayout, QHeaderView,
							 QTableWidget, QTableWidgetItem, QComboBox, 
							 QPushButton, QMenu, QApplication, QLineEdit,
							 QAbstractItemView, QButtonGroup, QProgressBar)
from PyQt5.QtGui import QFont, QIcon
from PyQt5.QtCore import Qt, QCoreApplication, QSize
from . import paths, parse_search_sites


class SearchTab(QWidget):
	"""Search class"""
	def __init__(self):
		super().__init__()
		self.create_widgets()
		
	def create_widgets(self):
		"""Create search widgets"""
		self.MAIN_URL = paths.search_tab_url
		self.PARSER = "html.parser"
		
	#vbox_main
		self.vbox_main = QVBoxLayout()
		self.vbox_main.setContentsMargins(1,1,1,1)
		self.setLayout(self.vbox_main)
		
		#font
		search_font = QFont()
		search_font.setPointSize(11)
		
	#combo_main
		self.combo_main = QComboBox()
		self.vbox_main.addWidget(self.combo_main)
		list_main = [
				(self.tr("Search video"), paths.search_icon,),
				(self.tr("Retro"), paths.retro_icon,),
			]
		for item in list_main:
			self.combo_main.addItem(QIcon(item[1]), item[0])
		self.combo_main.activated.connect(self.change_main_combo)
		
	#hbox_search
		self.hbox_search = QHBoxLayout()
		self.vbox_main.addLayout(self.hbox_search)
		#line_search
		self.line_search = QLineEdit()
		self.line_search.setPlaceholderText(self.tr("Search video"))
		self.line_search.setFont(search_font)
		self.line_search.setClearButtonEnabled(True)
		self.line_search.setMinimumHeight(paths.BUTTON_SIZE)
		self.line_search.setStyleSheet("QLineEdit{border:1px solid silver;}")
		self.line_search.returnPressed.connect(self.press_search)
		self.hbox_search.addWidget(self.line_search)
		#add button add station
		self.button_search_addmenu = QPushButton()
		icon = QIcon(paths.add_to_pls_icon)
		self.button_search_addmenu.setIconSize(QSize(25,25))
		self.button_search_addmenu.setIcon(icon)
		self.button_search_addmenu.setMinimumHeight(paths.BUTTON_SIZE)
		self.button_search_addmenu.setMinimumWidth(paths.BUTTON_SIZE)
		self.button_search_addmenu.setStyleSheet(paths.BUTTONS_STYLE)
		self.button_search_addmenu.setCursor(Qt.PointingHandCursor)
		self.hbox_search.addWidget(self.button_search_addmenu)
		#menu
		self.add_menu = QMenu()
		self.add_menu.addAction(QIcon(paths.add_file_icon), self.tr("Add selected item to PLS"), self.press_add_item_button)
		self.add_menu.addAction(QIcon(paths.pls_icon), self.tr("Add all items as playlist"), self.press_add_pls_button)
		self.add_menu.addAction(QIcon(paths.fav_icon), self.tr("Add selected item to Favs"), self.press_add_fav_button)
		#self.add_menu.addAction(QIcon(paths.info_icon), self.tr("Show info"), self.press_show_info_button)
		self.button_search_addmenu.setMenu(self.add_menu)
		
	#combo_translates
		self.combo_translates = QComboBox()
		self.vbox_main.addWidget(self.combo_translates)
		self.combo_translates.hide()
		self.combo_translates.addItem(self.tr("Choose voice:"))
		self.list_voices = [
				("Гаврилов", "http://megahit-online.guru/index.php?/forum/33-%D0%BE%D1%81%D1%82%D1%80%D0%BE%D1%81%D1%8E%D0%B6%D0%B5%D1%82%D0%BD%D0%BE%D0%B5-%D1%88%D0%B8%D0%BA%D0%B0%D1%80%D0%BD%D0%BE%D0%B5-%D0%BA%D0%B8%D0%BD%D0%BE-%D0%BE%D1%82-%D0%B3%D0%B0%D0%B2%D1%80%D0%B8%D0%BB%D0%BE%D0%B2%D0%B0/"),
				("Михалев", "http://megahit-online.guru/index.php?/forum/8-%D0%BA%D0%BE%D0%BC%D0%B5%D0%B4%D0%B8%D0%B8-%D1%82%D1%80%D0%B8%D0%BB%D0%BB%D0%B5%D1%80%D1%8B-%D0%B8-%D0%BC%D1%83%D0%BB%D1%8C%D1%82%D1%84%D0%B8%D0%BB%D1%8C%D0%BC%D1%8B-%D0%BE%D1%82-%D0%BC%D0%B8%D1%85%D0%B0%D0%BB%D0%B5%D0%B2%D0%B0/"),
				("Володарский", "http://megahit-online.guru/index.php?/forum/34-%D0%B3%D1%83%D0%BD%D0%B4%D0%BE%D1%81%D1%8B%D0%B9-%D0%BF%D0%B5%D1%80%D0%B5%D0%B2%D0%BE%D0%B4-%D0%BB%D0%B5%D0%B3%D0%B5%D0%BD%D0%B4%D0%B0%D1%80%D0%BD%D0%BE%D0%B3%D0%BE-%D0%B2%D0%BE%D0%BB%D0%BE%D0%B4%D0%B0%D1%80%D1%81%D0%BA%D0%BE%D0%B3%D0%BE/"),
				("Живов", "http://megahit-online.guru/index.php?/forum/38-%D0%BC%D0%B8%D1%80-%D0%BA%D1%80%D0%B0%D1%81%D0%B8%D0%B2%D0%BE%D0%B3%D0%BE-%D0%B8-%D0%B4%D0%B8%D0%BD%D0%B0%D0%BC%D0%B8%D1%87%D0%BD%D0%BE%D0%B3%D0%BE-%D0%BA%D0%B8%D0%BD%D0%BE-%D0%BE%D1%82-%D0%B6%D0%B8%D0%B2%D0%BE%D0%B2%D0%B0/"),
				("Горчаков", "http://megahit-online.guru/index.php?/forum/39-%D0%BA%D0%BE%D0%BC%D0%B5%D0%B4%D0%B8%D0%B8-%D1%82%D1%80%D0%B8%D0%BB%D0%BB%D0%B5%D1%80%D1%8B-%D0%B8-%D0%B2%D0%B5%D1%81%D1%82%D0%B5%D1%80%D0%BD%D1%8B-%D0%BE%D1%82-%D0%B3%D0%BE%D1%80%D1%87%D0%B0%D0%BA%D0%BE%D0%B2%D0%B0/"),
				("Сербин", "http://megahit-online.guru/index.php?/forum/40-%D0%BC%D0%B8%D1%80-%D1%81%D0%BE%D0%B2%D1%80%D0%B5%D0%BC%D0%B5%D0%BD%D0%BD%D0%BE%D0%B3%D0%BE-%D0%BA%D0%B8%D0%BD%D0%BE-%D0%B2-%D0%BF%D0%B5%D1%80%D0%B5%D0%B2%D0%BE%D0%B4%D0%B5-%D1%81%D0%B5%D1%80%D0%B1%D0%B8%D0%BD%D0%B0/"),
				("Визгунов", "http://megahit-online.guru/index.php?/forum/89-%D0%BE%D1%82-%D0%B7%D0%BE%D0%BC%D0%B1%D0%BE%D0%B0%D0%BF%D0%BE%D0%BA%D0%B0%D0%BB%D0%B8%D0%BF%D1%81%D0%B8%D1%81%D0%B0-%D0%B4%D0%BE-%D1%80%D0%BE%D0%BC%D0%B0%D0%BD%D1%82%D0%B8%D0%BA%D0%B8-%D0%BE%D1%82-%D0%B2%D0%B8%D0%B7%D0%B3%D1%83%D0%BD%D0%BE%D0%B2/"),
				("Дохалов", "http://megahit-online.guru/index.php?/forum/93-%D0%BE%D1%82%D0%BB%D0%B8%D1%87%D0%BD%D0%BE%D0%B5-%D0%BA%D0%B8%D0%BD%D0%BE-%D0%BD%D0%B0-%D1%80%D0%B0%D0%B7%D0%BD%D1%83%D1%8E-%D1%82%D0%B5%D0%BC%D0%B0%D1%82%D0%B8%D0%BA%D1%83-%D0%BE%D1%82-%D0%B4%D0%BE%D1%85%D0%B0%D0%BB%D0%BE%D0%B2%D0%B0/"),
				("Карцев", "http://megahit-online.guru/index.php?/forum/102-%D0%B7%D0%B2%D0%B5%D0%B7%D0%B4%D0%B0-%D0%BE%D1%81%D1%82%D1%80%D0%BE%D1%81%D1%8E%D0%B6%D0%B5%D1%82%D0%BD%D0%BE%D0%B3%D0%BE-%D0%BA%D0%BE%D0%BD%D1%82%D0%B5%D0%BD%D1%82%D0%B0-%D0%BF%D0%B5%D1%80%D0%B5%D0%B2%D0%BE%D0%B4%D1%8B-%D0%BA%D0%B0%D1%80%D1%86/"),
				("Дольский", "http://megahit-online.guru/index.php?/forum/88-%D1%85%D0%BE%D1%80%D1%80%D0%BE%D1%80%D1%8A-%D1%82%D1%80%D1%8D%D1%88-%D0%BA%D0%BE%D1%88%D0%BC%D0%B0%D1%80%D0%B8%D0%BA%D0%B8-%D0%B2-%D0%BF%D0%B5%D1%80%D0%B5%D0%B2%D0%BE%D0%B4%D0%B5-%D0%B4%D0%BE%D0%BB%D1%8C%D1%81%D0%BA%D0%BE%D0%B3%D0%BE/"),
				("Советская озвучка", "http://megahit-online.guru/index.php?/forum/7-%D1%84%D0%B8%D0%BB%D1%8C%D0%BC%D1%8B-%D1%81-%D1%81%D0%BE%D0%B2%D0%B5%D1%82%D1%81%D0%BA%D0%BE%D0%B9-%D0%BE%D0%B7%D0%B2%D1%83%D1%87%D0%BA%D0%BE%D0%B9-%D0%B4%D1%83%D0%B1%D0%BB%D1%8F%D0%B6%D0%BE%D0%BC/"),
			]
		for item in self.list_voices:
			self.combo_translates.addItem(QIcon("./icons/person_icon.png"), item[0])
		self.combo_translates.activated.connect(self.press_choose_voice)
		
	###table_widget
		self.table_widget = QTableWidget(0, 1)
		self.table_widget.setSelectionMode(QAbstractItemView.SingleSelection)
		self.table_widget.setHorizontalHeaderLabels(("Name",))
		self.table_widget.horizontalHeader().setSectionResizeMode(0, QHeaderView.Stretch)
		self.table_widget.verticalHeader().hide()
		self.table_widget.horizontalHeader().hide()
		self.table_widget.setShowGrid(True)
		self.table_widget.itemDoubleClicked.connect(self.press_search_item)
		self.vbox_main.addWidget(self.table_widget)
		
	#combo_pages
		self.combo_pages = QComboBox()
		self.combo_pages.hide()
		self.combo_pages.activated.connect(self.change_retro_page)
		self.vbox_main.addWidget(self.combo_pages)
		self.vbox_main.setAlignment(self.combo_pages, Qt.AlignCenter)
		
#################################################################################################

	def hide_sub_widgets(self):
		"""Hide sub widgets"""
		if self.combo_translates.isVisible():
			self.combo_translates.hide()

	def change_main_combo(self):
		"""Change main combo"""
		self.hide_sub_widgets()
		self.table_widget.setRowCount(0)
		index = self.combo_main.currentIndex()
		if index > 0:
			self.MAIN_URL = "http://megahit-online.guru/"
			if not self.combo_translates.isVisible():
				self.combo_translates.show()
		else:
			self.MAIN_URL = paths.search_tab_url
		#search line focus
		self.line_search.setFocus(True)
		
	def press_search(self):
		"""Press search"""
		text = self.line_search.text()
		if text:
			self.hide_sub_widgets()
			index = self.combo_main.currentIndex()
			self.table_widget.setRowCount(0)
			if index == 0:
				self.get_anwap_divs(text)
			if index == 1:
				if self.combo_pages.isVisible():
					self.combo_pages.hide()
				self.parse_retro(text)

	def add_one_row(self, name, url):
		"""Add one row"""
		#name
		fileNameItem = MyTableWidgetItem()
		fileNameItem.setIcon(QIcon(paths.video_icon))
		fileNameItem.setText(name)
		fileNameItem.setToolTip(name)
		fileNameItem.setFlags(fileNameItem.flags() ^ Qt.ItemIsEditable)
		fileNameItem.setTextAlignment(Qt.AlignVCenter | Qt.AlignLeft)
		fileNameItem.URL = url
		#set
		row = self.table_widget.rowCount()
		self.table_widget.insertRow(row)
		self.table_widget.setItem(row, 0, fileNameItem)
		
	def get_file_url(self, url):
		"""Get file url"""
		get_url = ""
		index = self.combo_main.currentIndex()
		if index == 0:
			get_url = parse_search_sites.get_anwap_film_url(url)
		if index == 1:
			page_url = url
			try:
				if page_url[-1] == "/":
					page_url = url[:len(url)-1]
			except:
				pass
			url_head, url_tail = os.path.split(page_url)
			url = os.path.join(url_head, urllib.parse.quote(url_tail))
			get_url = self.get_video_url(url)
		return get_url
		
####################################ADD MENU####################################################

	def press_add_pls_button(self):
		"""Press add all items as PLS"""
		if self.table_widget.rowCount() > 0:
			self.parent().parent().parent().pls_tab.clear()
			for i in range(0, self.table_widget.rowCount()):
				QCoreApplication.processEvents()
				current_item = self.table_widget.item(i,0)
				name = current_item.text()
				url = current_item.URL
				url = self.get_file_url(url)
				self.parent().parent().parent().pls_tab.set_row(name, url)
			self.parent().parent().parent().tabber.setCurrentIndex(1)
			
	def press_add_item_button(self):
		"""Press add one item to PLS"""
		current_item = self.table_widget.currentItem()
		if current_item:
			title = current_item.text()
			url = current_item.URL
			url = self.get_file_url(url)
			self.parent().parent().parent().pls_tab.set_row(title, url)
		
	def press_add_fav_button(self):
		"""Press add one item to FAV"""
		current_item = self.table_widget.currentItem()
		if current_item:
			name = current_item.text()
			url = current_item.URL
			url = self.get_file_url(url)
			self.parent().parent().parent().add_to_fav(name, url)
			
	def press_search_item(self, tableItem):
		"""Press search item"""
		name = tableItem.text()
		url = tableItem.URL
		get_url = self.get_file_url(url)
		self.parent().parent().parent().press_any_item(name, get_url)
				
######################################ANWAP################################################

	def get_post_params(self, text):
		"""Get anwap post params"""
		index = self.combo_main.currentIndex()
		if index == 0:
			post_params = {
				'search' : "Поиск",
				'slv' : text,
				'toch' : 1,
				'vid' : 1
			}
		if index == 1:
			post_params = {
				'query' : text,
				'search_app' : "forums"
			}
		return post_params

	def get_anwap_divs(self, text):
		"""Get anwap divs"""
		PARSER = "html.parser"
		post_params = self.get_post_params(text)
		post_args = urllib.parse.urlencode(post_params)
		binary_data = post_args.encode('utf-8')
		myopener = MyOpener()
		fp = myopener.open(self.MAIN_URL, binary_data)
		soup = BS(fp, PARSER)
		all_divs = soup.find_all("div", attrs={"class":"my_razdel film"})
		for item in all_divs:
			QCoreApplication.processEvents()
			name = item.find("div", attrs={"class":"namefilm"}).text.strip()
			href_item = item.a.get('href')
			if 'films' in href_item:
				url = paths.search_tab_start_url + href_item
				self.add_one_row(name, url)
		
######################################-RETRO-################################################

	def parse_retro(self, text):
		"""Parse retro"""
		url = "http://megahit-online.guru/search.php"
		params = self.get_post_params(text)
		post_args = urllib.parse.urlencode(params)
		binary_data = post_args.encode('utf-8')
		req = Request(url, headers={'User-Agent': 'Mozilla/5.0'})
		fp = urllib.request.urlopen(req, binary_data)
		soup = BS(fp, self.PARSER)
		all_divs = soup.find_all("td", attrs={"class":"col_c_forum"})
		if len(all_divs) > 0:
			for item in all_divs:
				try:
					QCoreApplication.processEvents()
					title = item.a.text.strip()
					href = item.a.get("href")
					self.add_one_row(title, href)
				except:
					pass
			
	def press_choose_voice(self):
		"""Press choose voice"""
		self.combo_pages.clear()
		index = self.combo_translates.currentIndex()
		if index > 0:
			url = self.list_voices[index - 1][1]
			self.parse_url(url)
			if not self.combo_pages.isVisible():
				self.combo_pages.show()
		else:
			self.table_widget.setRowCount(0)
			if self.combo_pages.isVisible():
				self.combo_pages.hide()
				
	def parse_url(self, url):
		"""Parse url"""
		self.table_widget.setRowCount(0)
		QCoreApplication.processEvents()
		req = Request(url, headers={'User-Agent': 'Mozilla/5.0'})
		webpage = urlopen(req).read()
		QCoreApplication.processEvents()
		soup = BS(webpage, self.PARSER)
		all_divs = soup.find_all("td", attrs={"class":"col_f_content "})
		if len(all_divs) > 0:
			for item in all_divs:
				if item.span.text == "Закреплено":
					continue
				QCoreApplication.processEvents()
				name = item.find('h4').a.span.text
				href = item.find('h4').a.get("href")
				self.add_one_row(name, href)
				QCoreApplication.processEvents()
			if self.combo_pages.count() <= 0:
				page_div = soup.find("ul", attrs={"class":"ipsList_inline left pages"})
				total_pages = re.search(r"totalPages:.*,", str(page_div)).group(0).split(" ")[1]
				total_pages = total_pages.replace(",", "")
				self.set_pages(total_pages)
				
	def set_pages(self, pages_num):
		"""Set pages num"""
		for num in range(1, int(pages_num) + 1):
			self.combo_pages.addItem(QIcon("./icons/page_icon.png"), str(num))

	def change_retro_page(self):
		"""Change retro page"""
		page_num = self.combo_pages.currentIndex()
		url = self.list_voices[self.combo_translates.currentIndex() - 1][1]
		if page_num > 0:
			url = url + "page-" + str(page_num + 1)
		self.parse_url(url)
		
	def get_video_url(self, url):
		"""Get video url"""
		try:
			QCoreApplication.processEvents()
			req = Request(url, headers={'User-Agent': 'Mozilla/5.0'})
			webpage = urlopen(req).read()
			QCoreApplication.processEvents()
			soup = BS(webpage, self.PARSER)
			#search_pattern = re.compile(r'Скачать фильм|Скачать видео|Скачать серию|Скачать мультфильм', re.I)
			search_pattern = re.compile(r'Скачать.*', re.I)
			get_url = soup.find("a", text=search_pattern).get("href")
			video_url = self.get_redirect_video_url(get_url)
		except:
			video_url = "None"
		return video_url
	
	def get_redirect_video_url(self, url):
		"""Get redirect video url"""
		QCoreApplication.processEvents()
		req = Request(url, headers={'User-Agent': 'Mozilla/5.0'})
		webpage = urlopen(req).read()
		QCoreApplication.processEvents()
		soup = BS(webpage, self.PARSER)
		href = re.search(r'innerHTML=".*"', str(soup)).group(0).split("\"")[1]
		redirect_video_url = re.search(r"href='.*'", str(href)).group(0).split("\'")[1]
		return redirect_video_url
	
###############################################################################################

class MyTableWidgetItem(QTableWidgetItem):
	"""My TableWidgetItem class"""
	def __init__(self):
		super().__init__()
		self.URL = None
		
###############################################################################################

class LetterButton(QPushButton):
	"""Letter button class"""
	def __init__(self):
		super().__init__()
		self.create_button()
		
	def create_button(self):
		"""Create Letterbutton widgets"""
		self.setFixedSize(25,20)
		self.setCursor(Qt.PointingHandCursor)
		self.setStyleSheet(paths.BUTTONS_STYLE)
		
################################################################################################

class MyOpener(FancyURLopener):
	version = ('Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/66.0.3359.170 Safari/537.36 OPR/53.0.2907.106')
