"""tvMain module"""
import os
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from . import paths, parseSite, tvpage

class TvMain(QWidget):
	"""TvMain class"""
	def __init__(self):
		super().__init__()
		self.create_widgets()
		
	def create_widgets(self):
		"""Create widgets"""
		#set ups
		self.current_date = QDate().currentDate().toString("yyyy-MM-dd")
		self.site_template = "http://www.vsetv.com/schedule_channel_{0}_day_{1}_ft_all.html"
		self.tv_icons_path = "./icons/tvicons/"
		
		#vbox main
		self.vbox_main = QVBoxLayout()
		self.vbox_main.setContentsMargins(0, 0, 0, 0)
		#self.vbox_main.setSpacing(2)
		self.setLayout(self.vbox_main)
		
	#hbox_tools
		self.hbox_tools = QHBoxLayout()
		self.vbox_main.addLayout(self.hbox_tools)
		#line_search
		self.line_search = QLineEdit()
		self.line_search.setClearButtonEnabled(True)
		self.line_search.setPlaceholderText(self.tr("search channel"))
		self.line_search.textChanged.connect(self.press_search)
		self.hbox_tools.addWidget(self.line_search)
		#
		self.hbox_tools.addStretch()
		#combo countries
		self.combo_countries = QComboBox()
		self.hbox_tools.addWidget(self.combo_countries)
		#self.vbox_main.setAlignment(self.combo_countries, Qt.AlignCenter)
		self.combo_countries.setIconSize(QSize(20,20))
		self.combo_countries.addItem(QIcon(paths.ukraine_flag), self.tr("Ukraine"))
		self.combo_countries.addItem(QIcon(paths.russia_flag), self.tr("Russia"))
		self.combo_countries.activated.connect(self.set_country)
		
		#list_widget
		self.list_widget = QListWidget()
		self.list_widget.setMovement(QListView.Snap)
		self.list_widget.setViewMode(QListView.IconMode)
		self.list_widget.setIconSize(QSize(60, 60))
		#self.list_widget.setWordWrap(True)
		self.list_widget.setUniformItemSizes(True)
		self.list_widget.setResizeMode(QListView.Adjust)
		self.list_widget.setGridSize(QSize(100, 100))
		self.list_widget.setStyleSheet("""
											QListWidget::item{
												margin:1px 1px 1px 1px;
												border:1px solid silver;
												border-radius:5px;
												max-width: 80px;
												min-width: 80px;
												max-height: 80px;
												min-height: 80px;
											}
											QListWidget::item::hover{
												border:1px solid orange;
												background:lightyellow;
											}
											QListWidget::item::selected{
												border:1px solid orange;
												background:lightyellow;
												color:orange;
											}
										""")
		self.list_widget.itemDoubleClicked.connect(self.press_double_item)
		self.vbox_main.addWidget(self.list_widget)
		#UKR
		search_params = ["Ukraine",]
		paths.CH_CURSOR.execute("SELECT * FROM channels WHERE ch_land=(?)", search_params)
		self.UKR_LIST = paths.CH_CURSOR.fetchall()
		#RUS
		search_params = ["Russia",]
		paths.CH_CURSOR.execute("SELECT * FROM channels WHERE ch_land=(?)", search_params)
		self.RUS_LIST = paths.CH_CURSOR.fetchall()
		#sort
		self.UKR_LIST.sort()
		self.RUS_LIST.sort()

		#label
		self.label_load = QLabel(self.tr("Opening"))
		self.label_load.setStyleSheet("""QLabel{
										border:1px solid silver; 
										background-color:lightblue;
										font-weight:bold;
									}""")
		self.label_load.hide()
		self.label_load.setMinimumHeight(30)
		self.label_load.setAlignment(Qt.AlignCenter)
		self.vbox_main.addWidget(self.label_load)
		
		#attribute
		self.setAttribute(Qt.WA_DeleteOnClose, True)
		
		#autostart
		self.check_countries()
######################################################################################

	def press_search(self):
		"""Press search"""
		text = self.line_search.text()
		current_country_index = self.combo_countries.currentIndex()
		if text:
			self.list_widget.clear()
			if current_country_index == 0:
				for item in self.UKR_LIST:
					if text.lower() in item[1].lower():
						url = self.site_template.format(item[2], self.current_date)
						image = os.path.join(self.tv_icons_path, item[3])
						self.add_one_item(item[1], url, image)
			if current_country_index > 0:
				for item in self.RUS_LIST:
					if text.lower() in item[1].lower():
						url = self.site_template.format(item[2], self.current_date)
						image = os.path.join(self.tv_icons_path, item[3])
						self.add_one_item(item[1], url, image)
		else:
			self.set_country(current_country_index)

	def check_countries(self):
		"""Check country"""
		current_index = self.combo_countries.currentIndex()
		self.set_country(current_index)
		
	def set_country(self, index):
		"""Set country"""
		self.list_widget.clear()
		if index == 0:
			for item in self.UKR_LIST:
				url = self.site_template.format(item[2], self.current_date)
				image = os.path.join(self.tv_icons_path, item[3])
				self.add_one_item(item[1], url, image)
		if index > 0:
			for item in self.RUS_LIST:
				url = self.site_template.format(item[2], self.current_date)
				image = os.path.join(self.tv_icons_path, item[3])
				self.add_one_item(item[1], url, image)
		self.line_search.setFocus(True)
		
	def add_one_item(self, title, url, image):
		"""Add one item"""
		self.item = MyItem()
		self.item.TITLE = title
		self.item.URL = url
		self.item.setText(title)
		self.item.setIcon(QIcon(image))
		self.list_widget.addItem(self.item)
				
	def press_double_item(self):
		"""Press double item"""
		QCoreApplication.processEvents()
		self.label_load.show()
		QCoreApplication.processEvents()
		#get title and url
		current_item = self.list_widget.currentItem()
		text = current_item.TITLE
		url = current_item.URL
		#get info
		info_list = parseSite.get_url(url)
		#open page
		self.tv_page = tvpage.Tvpage(url, info_list)
		self.parent().parent().parent().add_tab(self.tv_page, text)

########################################################

class MyItem(QListWidgetItem):
	"""My ListWidgetItem class"""
	def __init__(self):
		super().__init__()
		self.TITLE = None
		self.URL = None
