import datetime
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from . import parseSite

class Tvpage(QWidget):
	def __init__(self, url, info_list):
		super().__init__()
		self.url = url
		self.info_list = info_list
		self.create_widgets()
		
	def create_widgets(self):
		"""Create widgets"""
		self.CURRENT_ITEM = None
		#vbox main
		self.vbox_main = QVBoxLayout()
		self.setLayout(self.vbox_main)

	#weekdays
		self.hbox_weekdays = QHBoxLayout()
		self.vbox_main.addLayout(self.hbox_weekdays)

		date = QDate().currentDate()
		current_day = date.toString("yyyy-MM-dd")
		week_day = date.dayOfWeek()

		today = datetime.date.today()
		monday = today - datetime.timedelta(week_day - 1)

		self.BUTTON_GROUP = QButtonGroup()
		self.BUTTON_GROUP.setExclusive(True)
		for x in range(7):
			day = monday + datetime.timedelta(x)
			url = self.url.replace(current_day, str(day))
			name = QDate(day).toString("ddd")
			self.button = weekDayButton(url)
			self.button.setText(name)
			self.button.clicked.connect(self.press_weekday_button)
			self.hbox_weekdays.addWidget(self.button)
			self.BUTTON_GROUP.addButton(self.button)
			if x == week_day - 1:
				self.button.setChecked(True)
		
	#table
		self.table_view = QTableWidget(0, 2)
		self.table_view.setHorizontalHeaderLabels(("Time","Name",))
		self.table_view.setStyleSheet("QTableWidget::item{padding:5px;}")
		self.table_view.verticalHeader().hide()
		self.table_view.setShowGrid(True)
		self.vbox_main.addWidget(self.table_view)
		self.table_view.horizontalHeader().setSectionResizeMode(0, QHeaderView.ResizeToContents)
		self.table_view.horizontalHeader().setSectionResizeMode(1, QHeaderView.Stretch)
		
		#add rows
		for item in self.info_list:
			QCoreApplication.processEvents()
			if item[2] > 0:
				self.add_one_row(item[0], item[1], True)
			else:
				self.add_one_row(item[0], item[1], False)
		self.table_view.resizeRowsToContents()
		
		#scroll timer
		self.scrollTimer = QTimer()
		self.scrollTimer.setInterval(1000)
		self.scrollTimer.setSingleShot(True)
		self.scrollTimer.timeout.connect(self.scroll_item_start)
		self.scrollTimer.start()
		
################################################################################

	def add_one_row(self, time_text, name_text, stateItem):
		"""Add one row"""
		font = QFont()
		font.setBold(True)
		#time item
		timeItem = QTableWidgetItem(time_text)
		timeItem.setForeground(QColor("#2577fc"))
		timeItem.setFont(font)
		timeItem.setFlags(timeItem.flags() ^ Qt.ItemIsEditable ^ Qt.ItemIsSelectable)
		#channel item
		channelItem = QTableWidgetItem(name_text)
		channelItem.setFont(font)
		channelItem.setFlags(channelItem.flags() ^ Qt.ItemIsEditable ^ Qt.ItemIsSelectable)
		if stateItem:
			timeItem.setBackground(QColor("lightgreen"))
			channelItem.setBackground(QColor("lightgreen"))
			self.CURRENT_ITEM = timeItem
		#insert row
		row = self.table_view.rowCount()
		self.table_view.insertRow(row)
		self.table_view.setItem(row, 0, timeItem)
		self.table_view.setItem(row, 1, channelItem)
		
	def press_weekday_button(self):
		"""Press weekday button"""
		self.CURRENT_ITEM = None
		self.table_view.setRowCount(0)
		url = self.sender().URL
		info_list = parseSite.get_url(url)
		if len(info_list) > 0:
			for item in info_list:
				QCoreApplication.processEvents()
				if item[2] > 0:
					self.add_one_row(item[0], item[1], True)
				else:
					self.add_one_row(item[0], item[1], False)
		self.scrollTimer.start()
		
	def scroll_item_start(self):
		"""Scroll to current item"""
		if self.CURRENT_ITEM:
			self.table_view.scrollToItem(self.CURRENT_ITEM, QAbstractItemView.PositionAtCenter)
		
####################################################################################

	def closeEvent(self, event):
		"""Close event"""
		self.table_view.setRowCount(0)
		self.table_view.deleteLater()
		self.deleteLater()

#####################################################################################

class weekDayButton(QPushButton):
	"""weekDayButton class"""
	def __init__(self, url):
		super().__init__()
		self.URL = url
		self.create_widget()

	def create_widget(self):
		"""Create widgets"""
		self.setCheckable(True)
		self.setFixedHeight(25)
		self.setCursor(Qt.PointingHandCursor)
		self.setStyleSheet("""
								QPushButton{
									border:1px solid silver;
									border-radius:5px;
								}
								QPushButton::hover{
									border:1px solid #76a9fc;
								}
								QPushButton::checked{
									background-color:#cdd0d6; 
								}
			""")
