from PyQt5.QtWidgets import QWidget, QSizePolicy, QShortcut, QMenu
from PyQt5.QtCore import Qt, QPoint, QFileInfo
from PyQt5.QtGui import QKeySequence, QPalette, QIcon, QCursor, QBrush, QPixmap
from . import paths


class VideoWidget(QWidget):
	def __init__(self):
		super().__init__()
		self.setAcceptDrops(True)
		self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
		self.shortcutFull = QShortcut(self)
		self.shortcutFull.setKey(QKeySequence('Alt+Return'))
		self.shortcutFull.setContext(Qt.ApplicationShortcut)
		self.shortcutFull.activated.connect(self.handleFullScreen)
		#set black background
		pal = self.palette()
		pal.setColor(QPalette.Window, Qt.black)
		self.setPalette(pal)
		#set attributes
		self.setAttribute(Qt.WA_DontCreateNativeAncestors, True)
		self.setAttribute(Qt.WA_NativeWindow, True)
		self.setAttribute(Qt.WA_OpaquePaintEvent, True)

##############################################################################

	def mouseDoubleClickEvent(self, event):
		"""Mouse double click"""
		if event.button() == Qt.LeftButton:
			self.handleFullScreen()
		
	def handleFullScreen(self):
		"""Handle FullScreen"""
		if self.parent().parent().parent().isFullScreen():
			self.show_all_widgets()
			self.setCursor(Qt.ArrowCursor)
			self.parent().parent().parent().setWindowState(Qt.WindowActive)
		else: 
			self.hide_all_widgets()
			self.setCursor(Qt.BlankCursor)
			self.parent().parent().parent().setWindowState(Qt.WindowFullScreen)
			
	def show_all_widgets(self):
		"""Show all widgets"""
		self.parent().parent().parent().showNormal()
		self.parent().parent().control_widget.show()
		self.parent().parent().tabber.show()
		self.parent().parent().errors_label.show()
		self.parent().parent().label_programm.show()
		self.parent().parent().menu_main.show()
		
	def hide_all_widgets(self):
		"""Hide all widgets"""
		self.parent().parent().control_widget.hide()
		self.parent().parent().tabber.hide()
		self.parent().parent().errors_label.hide()
		self.parent().parent().label_programm.hide()
		self.parent().parent().menu_main.hide()
		
	def mouseReleaseEvent(self, event):
		"""Right mouse button click"""
		if event.button() == Qt.RightButton:
			if self.parent().parent().parent().isFullScreen():
				if self.parent().parent().control_widget.isVisible():
					self.parent().parent().control_widget.hide()
					self.parent().parent().tabber.hide()
					self.parent().parent().menu_main.hide()
					self.setCursor(Qt.BlankCursor)
				else:
					self.parent().parent().control_widget.show()
					self.parent().parent().tabber.show()
					self.parent().parent().menu_main.show()
					self.setCursor(Qt.ArrowCursor)
		
	def press_volume_mute(self):
		"""Press volume mute"""
		self.parent().parent().press_mute_button()
		
	def press_play_button(self):
		"""Press play"""
		self.parent().parent().press_play_button()

	def press_button_pause(self):
		"""Press pause"""
		self.parent().parent().press_button_pause()

	def press_stop(self):
		"""Press pause"""
		self.parent().parent().press_stop()

	def press_next_file(self):
		"""Press next"""
		self.parent().parent().press_next_file()

	def press_prev_file(self):
		"""Press prev"""
		self.parent().parent().press_prev_file()

###########################################################################################

	def wheelEvent(self, event):
		"""Wheel event"""
		pd = event.pixelDelta()
		ad = event.angleDelta()
			
		value = self.parent().parent().control_widget.volume_slider.sliderPosition()
			
		if pd.y() > 0 or ad.y() > 0:
			value += 2
			if value >= 100:
				value = 100
		if pd.y() < 0 or ad.y() < 0:
			value -= 2
			if value <= 0:
				value = 0
					
		self.parent().parent().control_widget.volume_slider.setSliderPosition(value)
		self.parent().parent().volume_change(value)

################################################################################################
