import os
import sys
import time
import datetime
import random
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from . import (config, pls, add_station, get_info, menu_bar,
			   filestab, searchTab, videoWidget, favtab, paths, 
				controlWidget, ytab, bookmarksTab, kinokrad, mpv,
				tv)


class Widgets(QWidget):
	def __init__(self):
		super().__init__()
		self.setMouseTracking(True)
		self.push_stop = False
		self.CURRENT_URL = ""
		self.slider_state = False
		self.CURRENT_CH_ID = None
		self.CURRENT_CH_NAME = None
		self.create_widgets()


	def create_widgets(self):
		#config file
		self.settings = QSettings(paths.config_path, QSettings.IniFormat)
		#vbox_main
		self.vbox_main = QVBoxLayout()
		self.vbox_main.setContentsMargins(0, 0, 0, 0)
		self.setLayout(self.vbox_main)
		
#create menu_bar
		self.menu_main = menu_bar.Menu_bar()
		#actions
		self.menu_main.open_file_action.triggered.connect(self.press_menu_open_file)
		self.menu_main.open_folder_action.triggered.connect(self.press_menu_open_folder)
		self.menu_main.open_pls_action.triggered.connect(self.press_menu_open_pls_folder)
		self.menu_main.close_app_action.triggered.connect(self.press_menu_close)
		#playback
		self.menu_main.play_action.triggered.connect(self.press_play_button)
		self.menu_main.pause_action.triggered.connect(self.press_button_pause)
		self.menu_main.stop_action.triggered.connect(self.press_stop)
		self.menu_main.next_file_action.triggered.connect(self.press_next_file)
		self.menu_main.prev_file_action.triggered.connect(self.press_prev_file)
		self.menu_main.record_action.triggered.connect(self.press_record_button)
		#video
		self.menu_main.fullscreen_action.triggered.connect(self.toggle_fullscreen)
		self.menu_main.deinterlace_action.toggled.connect(self.toggle_deinterlace)
		self.menu_main.zoom_action.toggled.connect(self.toggle_video_zoom)
		self.menu_main.screenshot_action.triggered.connect(self.take_screenshot)
		self.menu_main.action_no_aspect.toggled.connect(self.change_aspect)
		self.menu_main.action_aspect_4_3.toggled.connect(self.change_aspect)
		self.menu_main.action_aspect_16_9.toggled.connect(self.change_aspect)
		#subtitles
		self.menu_main.open_subtitles_action.triggered.connect(self.press_open_subtitles)
		#tools
		self.menu_main.add_stream_action.triggered.connect(self.press_menu_add_stream)
		self.menu_main.create_pls_action.triggered.connect(self.press_menu_create_pls)
		self.menu_main.tvgid_action.triggered.connect(self.press_menu_tvgid)
		self.menu_main.backup_action.triggered.connect(self.press_menu_backup)
		#add tabs
		self.menu_main.kinokrad_action.triggered.connect(self.press_menu_kinokrad)
		#about
		self.menu_main.guide_action.triggered.connect(self.press_menu_guide)
		self.menu_main.video_review_action.triggered.connect(self.press_menu_video_review)
		self.menu_main.about_action.triggered.connect(self.press_menu_about)
		self.vbox_main.addWidget(self.menu_main)

#hbox main
		self.hbox_line_main = QHBoxLayout()
		self.hbox_line_main.setContentsMargins(0, 0, 0, 0)
		self.vbox_main.addLayout(self.hbox_line_main)

#tabber
		self.tabber = QTabWidget()
		self.tabber.setFixedWidth(300)
		self.tabber.setIconSize(QSize(24, 24))
		self.tabber.setTabPosition(QTabWidget.West)
		self.hbox_line_main.addWidget(self.tabber)

	#add FAVS
		self.fav_tab = favtab.FavTab()
		self.add_new_tab(self.fav_tab, QIcon(paths.fav_icon), '')
		
	#add PLS
		self.pls_tab = pls.Pls()
		self.add_new_tab(self.pls_tab, QIcon(paths.pls_icon), '')

	#add FILES TAB
		self.files_tab = filestab.FilesTab()
		self.files_tab.treeview.doubleClicked.connect(self.press_files_item)
		self.add_new_tab(self.files_tab, QIcon(paths.folder_icon), '')
		
	#add YTAB
		self.ytab = ytab.YTab()
		self.add_new_tab(self.ytab, QIcon(paths.yt_icon), "")
		
	#add SEARCHTAB
		self.search_tab = searchTab.SearchTab()
		self.add_new_tab(self.search_tab, QIcon(paths.search_icon), '')
		
	#add MarksTab
		self.bookmarks_tab = bookmarksTab.BookmarksTab()
		self.add_new_tab(self.bookmarks_tab, QIcon(paths.bookmarks_icon), '')

	#add CONFIG TAB
		self.config_tab = config.Config()
		self.config_tab.button_check.clicked.connect(self.press_check_channel)
		self.config_tab.slider_brightness.valueChanged.connect(self.change_brightness)
		self.config_tab.slider_contrast.valueChanged.connect(self.change_contrast)
		self.config_tab.slider_saturation.valueChanged.connect(self.change_saturation)
		self.config_tab.slider_hue.valueChanged.connect(self.change_hue)
		self.config_tab.default_button.clicked.connect(self.press_default_button)
		self.add_new_tab(self.config_tab, QIcon(paths.config_icon), '')

#rigth_side
		self.rightWidget = QWidget()
		self.rightWidget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
		self.hbox_line_main.addWidget(self.rightWidget)
		self.vbox_rigth = QVBoxLayout()
		self.vbox_rigth.setContentsMargins(0, 0, 0, 0)
		self.rightWidget.setLayout(self.vbox_rigth)
		
		#error label
		self.hbox_main_label = QHBoxLayout()
		self.hbox_main_label.setContentsMargins(0, 0, 0, 0)
		self.vbox_rigth.addLayout(self.hbox_main_label)
		label_font = QFont()
		label_font.setBold(True)
		self.errors_label = QLabel()
		self.errors_label.setStyleSheet("QLabel{color:#0061ff;}")
		self.errors_label.setWordWrap(True)
		self.errors_label.setAlignment(Qt.AlignLeft)
		self.errors_label.setFont(label_font)
		self.hbox_main_label.addWidget(self.errors_label)
		#label programm
		self.label_programm = QLabel()
		self.label_programm.setWordWrap(True)
		self.label_programm.setAlignment(Qt.AlignLeft)
		self.label_programm.setFont(label_font)
		self.hbox_main_label.addWidget(self.label_programm)

		#video widget
		self.video_widget = videoWidget.VideoWidget()
		self.vbox_rigth.addWidget(self.video_widget)
		self.player = mpv.MPV(
								wid=str(int(self.video_widget.winId())),
								ytdl=True
							)
		self.player._set_property('osd-align-x', 'center')
		self.player._set_property('osd-color', '#1ece00')
		self.player._set_property('input-cursor', False)
		self.player._set_property('cursor-autohide', False)
		self.player._set_property('image-display-duration', 'inf')
		screenshots_folder = QStandardPaths.writableLocation(QStandardPaths.PicturesLocation)
		self.player._set_property('screenshot-directory', screenshots_folder)
		#print(self.player._get_property('audio-device-list'))

#control widget
		self.control_widget = controlWidget.ControlWidget()
		self.control_widget.button_prev.clicked.connect(self.press_prev_file)
		self.control_widget.position_slider.sliderReleased.connect(self.hand_pos_changed)
		self.control_widget.position_slider.sliderPressed.connect(self.set_slider_pressed)
		self.control_widget.position_slider.valueChanged.connect(self.move_time)
		self.control_widget.button_next.clicked.connect(self.press_next_file)
		self.control_widget.button_hide_left.clicked.connect(self.press_hide_left_button)
		self.control_widget.button_play.clicked.connect(self.press_play_button)
		self.control_widget.button_stop.clicked.connect(self.press_stop)
		self.control_widget.button_pause.clicked.connect(self.press_button_pause)
		self.control_widget.volume_slider.valueChanged.connect(self.volume_change)
		self.control_widget.button_fullscreen.clicked.connect(self.press_button_fullscreen)
		self.control_widget.button_mute.clicked.connect(self.press_mute_button)
		self.control_widget.button_tvgid.clicked.connect(self.press_tvgid_button)
		self.control_widget.button_minus_time.clicked.connect(self.press_minus_time)
		self.control_widget.button_plus_time.clicked.connect(self.press_plus_time)
		self.vbox_rigth.addWidget(self.control_widget)
		
		#autostart
		self.player._set_property('volume', 30)
		self.create_shortcuts()
		self.set_last_pls_url()
		self.set_last_tab()
		self.press_default_button()
		self.play_default()
################################################################################

		@self.player.property_observer('time-pos')
		def time_observer(_name, value):
			"""Time position"""
			if value:
				if self.player._get_property('duration'):
					time_current = time.strftime('%H:%M:%S', time.gmtime(value))
					self.control_widget.label_pos_current.setText(time_current)
					if self.slider_state == False:
						self.control_widget.position_slider.setValue(int(value))
			
		@self.player.property_observer('duration')
		def duration_observer(_name, value):
			"""Duration"""
			if value:
				self.control_widget.position_slider.setRange(0, int(value))
				self.control_widget.label_pos_last.setText(time.strftime('%H:%M:%S', time.gmtime(value)))
			else:
				self.control_widget.position_slider.setRange(0, 0)
				self.control_widget.label_pos_last.clear()
			
		@self.player.property_observer('percent-pos')
		def percent_pos_observer(_name, value):
			if value:
				if value >= 99.9:
					self.change_mode_play()
				
################################################################################
	
	def set_last_pls_url(self):
		"""Set last pls url"""
		text = self.settings.value("config/last_pls_url")
		if text:
			self.pls_tab.pls_line.setText(text)
			
	def set_last_tab(self):
		"""Set last tab num"""
		current_tab = self.settings.value("config/last_tab")
		current_tab = int(current_tab)
		self.tabber.setCurrentIndex(current_tab)
		
#################################################################################

	def play_default(self):
		"""Play default video"""
		self.player_play('./icons/tv_512.jpg')
						
	def set_name_channel(self, name, *args):
		"""Set channel info"""
		self.errors_label.setText((name[:38]).upper())
		current_url = self.CURRENT_URL
		if self.CURRENT_URL:
			if not self.player._get_property('duration'):
				title = get_info.get_url(name)
				self.label_programm.setText(title)
			else:
				self.label_programm.clear()
		
#####################################################################################################
	def press_button_fullscreen(self):
		"""Press fullscreen button"""
		self.video_widget.handleFullScreen()
		
	def press_mute_button(self):
		"""Press mute button"""
		if self.player._get_property('mute'):
			self.change_mute_state(False)
		else:
			self.change_mute_state(True)
			
	def change_mute_state(self, mute):
		"""Change mute state"""
		if mute:
			self.control_widget.button_mute.setIcon(QIcon(paths.volume_mute_icon))
			self.player._set_property('mute', True)
			self.player.command('show-text', 'mute')
		else:
			self.control_widget.button_mute.setIcon(QIcon(paths.volume_icon))
			self.player._set_property('mute', False)
			self.player.command('show-text', 'unmute')
			
	def press_tvgid_button(self):
		"""Create and launch TVGid win"""
		self.tv_win = tv.Tvgid()
		self.tv_win.show()
		self.tv_win.main_tab.line_search.setFocus(True)
		
###################################TABBER#########################################

	def press_any_item(self, title, url, *args):
		"""Press any item from any tab"""
		if url:
			self.errors_label.clear()
			self.player_play(url)
			self.set_name_channel(title)
		
	def press_files_item(self, index):
		"""Press files item"""
		current_path = self.files_tab.treeview.model().filePath(index)
		if os.path.isfile(current_path):
			self.errors_label.clear()
			if QFileInfo(current_path).suffix() in paths.VIDEO_LIST:
				self.player_play(current_path)
				title = os.path.split(current_path)[1]
				self.set_name_channel(title)
			if QFileInfo(current_path).suffix() in paths.PLS_LIST:
				self.tabber.setCurrentIndex(1)
				self.pls_tab.checking_pls(current_path)
			
####################################Shortcuts#######################################

	def press_change_tab(self):
		"""Change tab"""
		current_index = self.tabber.currentIndex()
		current_index += 1
		if current_index == self.tabber.count():
			current_index = 0
		self.tabber.setCurrentIndex(current_index)

	def press_appsleep(self):
		"""Press Z button for sleep"""
		self.press_button_pause()
		self.parent().showMinimized()
		
	def keyReleaseEvent(self, event):
		if event.nativeScanCode() == 59:
			self.press_minus_time()
		if event.nativeScanCode() == 60:
			self.press_plus_time()
			
	def press_minus_time(self):
		"""Press minus time"""
		if self.player._get_property('pause') == False:
			if self.player._get_property('duration'):
				if int(self.player._get_property('duration')) > 0:
					current_position = int(self.player._get_property('time-pos'))
					current_position -= 10
					self.player._set_property('time-pos', current_position)
					
	def press_plus_time(self):
		"""Press plus time"""
		if self.player._get_property('pause') == False:
			if self.player._get_property('duration'):
				if int(self.player._get_property('duration')) > 0:
					current_position = int(self.player._get_property('time-pos'))
					current_position += 15
					self.player._set_property('time-pos', current_position)
			
	def press_button_left(self):
		value = (self.control_widget.volume_slider.sliderPosition() - 2)
		if value <= 0:
			value = 0
		self.control_widget.volume_slider.setSliderPosition(value)
		self.volume_change(value)
		
	def press_button_right(self):
		value = (self.control_widget.volume_slider.sliderPosition() + 2)
		if value >= 100:
			value = 100
		self.control_widget.volume_slider.setSliderPosition(value)
		self.volume_change(value)

	def press_button_up(self):
		value = (self.config_tab.slider_brightness.sliderPosition() + 2)
		if value >= 100:
			value = 100
		self.config_tab.slider_brightness.setSliderPosition(value)
		self.change_brightness(value)
		
	def press_button_down(self):
		value = (self.config_tab.slider_brightness.sliderPosition() - 2)
		if value <= -100:
			value = -100
		self.config_tab.slider_brightness.setSliderPosition(value)
		self.change_brightness(value)

##################################ADJUST VIDEO##############################################
	def change_brightness(self, value):
		self.player._set_property('brightness', value)
		self.config_tab.set_brightness_text(value)

	def change_contrast(self, value):
		self.player._set_property('contrast', value)
		self.config_tab.set_contrast_text(value)

	def change_saturation(self, value):
		self.player._set_property('saturation', value)
		self.config_tab.set_saturation_text(value)

	def change_hue(self, value):
		self.player._set_property('hue', value)
		self.config_tab.set_hue_text(value)

	def press_default_button(self):
		value = 0
		self.change_brightness(value)
		self.change_contrast(value)
		self.change_saturation(value)
		self.change_hue(value)
		self.config_tab.default_values()
				
####################################################################################

	def ShowError(self):
		self.errors_label.clear()
		self.errors_label.setText("Error: " + self.player.errorString())
		
	def press_hide_left_button(self):
		if self.tabber.isVisible():
			self.tabber.hide()
		else:
			self.tabber.show()
			
	def add_to_fav(self, name, url):
		"""Add to FAV"""
		self.open_add_win(name, url)
			
	def open_add_win(self, name, url):
		self.add_win = add_station.Add_Station(self)
		self.add_win.set_name_url(name, url)
	
	def volume_change(self, value):
		"""Change volume"""
		self.TIME_SHOW = 3
		if value:
			if self.player._get_property('mute') == True:
				self.change_mute_state(False)
		else:
			if self.player._get_property('mute') == False:
				self.change_mute_state(True)
		volume_text = 'Volume: {0}'.format(value)
		self.player.command('show-text', volume_text)
		self.player._set_property('volume', value)
		self.control_widget.label_volume.setText(self.tr("Volume:") + str(self.control_widget.volume_slider.sliderPosition()))
		
	def press_check_channel(self):
		"""Check media"""
		text = self.config_tab.entry_check.text()
		if text:
			self.player_play(text)

	def player_play(self, url):
		"""Player play"""
		try:
			if self.player._get_property('record-file'):
				self.player._set_property('record-file', '')
		except:
			pass
		self.player.command('stop')
		self.push_stop = False
		self.CURRENT_URL = url
		self.player.play(url)
		self.player._set_property('pause', False)
		self.jump_time_position()

	def change_position_time(self, position):
		"""Change position time"""
		if position > 0:
			if int(self.player._get_property('duration')) > 0 and self.slider_state == False:
				self.control_widget.position_slider.setValue(position)
			current_time = time.strftime('%H:%M:%S', time.gmtime(position))
			self.control_widget.label_pos_current.setText(str(current_time))

	def hand_pos_changed(self):
		value = self.control_widget.position_slider.value()
		self.change_position_time(value)
		if self.player._get_property('time-pos'):
			self.player._set_property('time-pos', value)
		self.slider_state = False
		self.control_widget.label_move_time.hide()
		
	def move_time(self, position):
		if self.slider_state == True:
			current_time = time.strftime('%H:%M:%S', time.gmtime(position))
			self.control_widget.label_move_time.show()
			self.control_widget.label_move_time.setText(str(current_time))
	
	def set_slider_pressed(self):
		self.slider_state = True
		
	def jump_time_position(self):
		"""Jump time"""
		jump_time = self.config_tab.spinbox_jump_time.value()
		current_position = 0
		current_position += jump_time
		self.rewind(current_position)
			
	def rewind(self, time_seconds):
		"""Rewind video"""
		self.player._set_property('start', time_seconds)

	def press_button_pause(self):
		"""Press pause"""
		if self.player._get_property('pause'):
			self.player._set_property('pause', False)
		else:
			self.player._set_property('pause', True)
		
	def press_stop(self):
		"""Press stop"""
		try:
			if self.player._get_property('record-file'):
				self.player._set_property('record-file', '')
		except:
			pass
		self.player.command('stop')
		self.control_widget.label_pos_current.clear()
		self.control_widget.label_pos_last.clear()
		self.errors_label.clear()
		self.label_programm.clear()
		self.push_stop = True
		self.play_default()

	def press_play_button(self):
		"""Press play"""
		self.player._set_property('pause', False)
		self.player.command('show-text', 'Play')

##########################################################################################

	def change_mode_play(self):
		"""Change mode play"""
		current_mode = self.control_widget.combo_playmodes.currentText().lower()
		mode_list = ["next", "shuffle",]
		if current_mode == "stop":
			self.press_stop()
		if current_mode == "replay":
			self.player_play(self.CURRENT_URL)
		if current_mode in mode_list:
			self.press_next_file()
		
	def press_next_file(self):
		"""Press next file"""
		last_pls_item_mode = False
		pls_row_count = self.pls_tab.model.rowCount()
		if pls_row_count > 0:
			current_index = self.pls_tab.table_view.currentIndex()
			if current_index:
				if self.control_widget.combo_playmodes.currentText().lower() == "shuffle":
					row_num = random.randrange(0, pls_row_count)
				else:
					row_num = current_index.row()
					row_num += 1
					if row_num == pls_row_count:
						row_num = 0
				self.choose_next_file(row_num)
				
	def choose_next_file(self, row_num):
		"""Choose next file"""
		next_item = self.pls_tab.sort_model.index(row_num, 0)
		self.pls_tab.press_pls_item(next_item)
		self.pls_tab.table_view.setCurrentIndex(next_item)
		
	def press_prev_file(self):
		"""Press prev file"""
		if self.pls_tab.model.rowCount() > 0:
			current_index = self.pls_tab.table_view.currentIndex()
			if current_index:
				row_num = current_index.row()
				row_num -= 1
				if row_num <= 0:
					row_num = 0
				self.choose_next_file(row_num)
				
############################################################
	def add_new_tab(self, widget, icon, name):
		"""Add new tab"""
		self.tabber.addTab(widget, icon, "")

###################################################################

	def create_shortcuts(self):
		"""Create shortcuts"""
		self.shortcutChangeTab = QShortcut(self)
		self.shortcutChangeTab.setKey(QKeySequence('Ctrl+Tab'))
		self.shortcutChangeTab.setContext(Qt.ApplicationShortcut)
		self.shortcutChangeTab.activated.connect(self.press_change_tab)
		
		self.shortcutAppSleep = QShortcut(self)
		self.shortcutAppSleep.setKey(QKeySequence('Z'))
		self.shortcutAppSleep.setContext(Qt.ApplicationShortcut)
		self.shortcutAppSleep.activated.connect(self.press_appsleep)

		self.shortcutPause = QShortcut(self)
		self.shortcutPause.setKey(QKeySequence('Space'))
		self.shortcutPause.setContext(Qt.ApplicationShortcut)
		self.shortcutPause.activated.connect(self.press_button_pause)
		
		self.shortcutLeft = QShortcut(self)
		self.shortcutLeft.setKey(QKeySequence('Down'))
		self.shortcutLeft.setContext(Qt.ApplicationShortcut)
		self.shortcutLeft.activated.connect(self.press_button_left)
		
		self.shortcutRight = QShortcut(self)
		self.shortcutRight.setKey(QKeySequence('Up'))
		self.shortcutRight.setContext(Qt.ApplicationShortcut)
		self.shortcutRight.activated.connect(self.press_button_right)
		
		self.shortcutUp = QShortcut(self)
		self.shortcutUp.setKey(QKeySequence('Right'))
		self.shortcutUp.setContext(Qt.ApplicationShortcut)
		self.shortcutUp.activated.connect(self.press_button_up)
		
		self.shortcutDown = QShortcut(self)
		self.shortcutDown.setKey(QKeySequence('Left'))
		self.shortcutDown.setContext(Qt.ApplicationShortcut)
		self.shortcutDown.activated.connect(self.press_button_down)
		
		self.shortcutMute = QShortcut(self)
		self.shortcutMute.setKey(QKeySequence('M'))
		self.shortcutMute.setContext(Qt.ApplicationShortcut)
		self.shortcutMute.activated.connect(self.press_mute_button)

##############################################################################
		
	def toggle_fullscreen(self):
		"""Toggle fullscreen"""
		self.video_widget.handleFullScreen()
		
	def toggle_deinterlace(self, state):
		"""Toggle deinterlace"""
		if state:
			self.player._set_property('deinterlace', True)
		else:
			self.player._set_property('deinterlace', False)
		
	def press_menu_guide(self):
		"""Press menu guide"""
		self.config_tab.press_howto_button()

	def press_menu_video_review(self):
		"""Press menu video review"""
		self.config_tab.press_howto_video_button()

	def press_menu_about(self):
		"""Press menu about"""
		self.config_tab.press_about_button()

	def press_menu_open_file(self):
		"""Press menu open file"""
		lists = "*.mp4 *.flv *.avi *.mpg *.mpeg *.mkv *.wmv *.webm *.mp3 *.ogg *.flac *.wav *.ac3"
		get_list_files = QFileDialog.getOpenFileNames(self, self.tr("Open media file(s)"), 
			QDir.homePath(), "({0})".format(lists))[0]
		if len(get_list_files) > 0:
			for item in get_list_files:
				self.files_tab.add_one_file_to_pls(item)
			current_tab_index = self.tabber.currentIndex()
			if current_tab_index != 1:
				self.tabber.setCurrentIndex(1)

	def press_menu_open_folder(self):
		"""Press menu open folder"""
		get_dir = QFileDialog.getExistingDirectory(self, self.tr("Choose directory"), QDir.homePath())
		if os.path.exists(get_dir):
			self.files_tab.add_whole_folder(get_dir)
			current_tab_index = self.tabber.currentIndex()
			if current_tab_index != 1:
				self.tabber.setCurrentIndex(1)

	def press_menu_close(self):
		"""Press menu close"""
		self.parent().close()
			
	def press_menu_add_stream(self):
		"""Press menu add stream"""
		self.fav_tab.press_add_button()
		
	def press_menu_create_pls(self):
		"""Press menu create PLS"""
		self.pls_tab.press_create_pls()
		
	def press_menu_tvgid(self):
		"""Press menu tvgid"""
		self.press_tvgid_button()

	def press_menu_backup(self):
		"""Press backup button"""
		from . import backup_win
		self.backup_win = backup_win.Backup_win()
		self.backup_win.show()
		
	def press_menu_open_pls_folder(self):
		"""Press menu open PLS folder"""
		self.pls_tab.press_open_manual()
		
	def press_menu_kinokrad(self):
		"""Press menu kinokrad"""
		self.kinokrad_tab = kinokrad.Kinokrad(self)
		self.add_new_tab(self.kinokrad_tab, QIcon('./icons/kinokrad_icon.png'), '')
		self.tabber.setCurrentIndex(self.tabber.count()-1)
			
	def toggle_video_zoom(self, state):
		"""Toggle zoom"""
		if state:
			self.player._set_property('panscan', 1.0)
		else:
			self.player._set_property('panscan', 0.0)
			
	def press_open_subtitles(self):
		"""Press open subtitles"""
		open_file = QFileDialog.getOpenFileName(self, self.tr("Open subtitles"), 
			QDir.homePath(), "Subtitles (*.srt)", options = QFileDialog.DontUseNativeDialog)
		if open_file[0]:
			self.player.command('sub-add', open_file[0])
			
	def press_record_button(self):
		"""Record"""
		time_current = time.strftime('%Y%m%d_%H%M%S', time.gmtime())
		file_name = "zvvtv_{0}_record.ts".format(time_current)
		try:
			video_path = QStandardPaths.writableLocation(QStandardPaths.MoviesLocation)
		except:
			video_path = QDir.homePath()
		video_file_path = os.path.join(video_path, file_name)
		try:
			self.player._set_property('record-file', video_file_path)
		except:
			pass
		
	def take_screenshot(self):
		"""Take screenshot"""
		self.player._set_property('screenshot-template', 'zvvtv-shot%n')
		self.player.screenshot()

	def change_aspect(self):
		"""Change aspect"""
		aspect_text = self.sender().text()
		if '16:9' in aspect_text.lower():
			self.player._set_property('video-aspect', '16:9')
		elif '4:3' in aspect_text.lower():
			self.player._set_property('video-aspect', '4:3')
		else:
			self.player._set_property('video-aspect', -1)
