"""YouTube module"""
from PyQt5.QtWidgets import (QWidget, QVBoxLayout, QHBoxLayout, 
							 QLineEdit, QPushButton, QMenu, 
							 QTableWidget, QTableWidgetItem, QAbstractItemView,
							 QHeaderView)
from PyQt5.QtGui import QIcon
from PyQt5.QtCore import Qt, QCoreApplication, QSettings, QSize
from apiclient.discovery import build
from apiclient.errors import HttpError
from oauth2client.tools import argparser
from . import paths


class YTab(QWidget):
	"""YT class"""
	def __init__(self):
		super().__init__()
		self.create_widgets()
		
	def create_widgets(self):
		"""Create widgets"""
		self.config_file = QSettings(paths.config_path, QSettings.IniFormat)
		self.DEVELOPER_KEY = "AIzaSyBdboJQUhSTxWcRSrM0yS0P_MoUGGMc1t8"
		self.YOUTUBE_API_SERVICE_NAME = "youtube"
		self.YOUTUBE_API_VERSION = "v3"
		#vbox_main
		self.vbox_main = QVBoxLayout()
		self.vbox_main.setContentsMargins(1,1,1,1)
		self.setLayout(self.vbox_main)
		
	#line search
		self.hbox_search = QHBoxLayout()
		self.vbox_main.addLayout(self.hbox_search)
		#line
		self.line_search = QLineEdit()
		self.line_search.setPlaceholderText(self.tr("Search youtube video"))
		self.line_search.setClearButtonEnabled(True)
		self.line_search.setMinimumHeight(paths.BUTTON_SIZE)
		self.line_search.returnPressed.connect(self.press_search)
		self.hbox_search.addWidget(self.line_search)
		#add button add station
		self.button_add_plsStation = QPushButton(QIcon(paths.add_to_pls_icon), "")
		icon = QIcon(paths.add_to_pls_icon)
		self.button_add_plsStation.setIconSize(QSize(25, 25))
		self.button_add_plsStation.setIcon(icon)
		self.button_add_plsStation.setMinimumHeight(paths.BUTTON_SIZE)
		self.button_add_plsStation.setMinimumWidth(paths.BUTTON_SIZE)
		self.button_add_plsStation.setStyleSheet(paths.BUTTONS_STYLE)
		self.button_add_plsStation.setCursor(Qt.PointingHandCursor)
		self.hbox_search.addWidget(self.button_add_plsStation)
		#menu
		self.yt_menu = QMenu()
		self.yt_menu.addAction(QIcon(paths.add_file_icon), self.tr("Add selected item to PLS"), self.press_add_item_button)
		self.yt_menu.addAction(QIcon(paths.pls_icon), self.tr("Add all items as playlist"), self.press_add_pls_button)
		self.yt_menu.addAction(QIcon(paths.fav_icon), self.tr("Add selected item to Favs"), self.press_add_fav_button)
		self.button_add_plsStation.setMenu(self.yt_menu)
		
	#table widget
		self.table_widget = QTableWidget(0,1)
		self.vbox_main.addWidget(self.table_widget)
		self.table_widget.setShowGrid(True)
		self.table_widget.verticalHeader().hide()
		self.table_widget.horizontalHeader().hide()
		self.table_widget.setSelectionBehavior(QAbstractItemView.SelectRows)
		self.table_widget.setSelectionMode(QAbstractItemView.SingleSelection)
		self.table_widget.horizontalHeader().setSectionResizeMode(0, QHeaderView.Stretch)
		self.table_widget.itemDoubleClicked.connect(self.press_yt_item)
		
##################################################################################################

	def press_search(self):
		"""Press search"""
		search_text = self.line_search.text()
		if search_text:
			self.table_widget.setRowCount(0)
			self.search_videos(search_text)

	def search_videos(self, text):
		"""Search YT videos"""
		youtube = build(self.YOUTUBE_API_SERVICE_NAME, 
						self.YOUTUBE_API_VERSION,
						developerKey=self.DEVELOPER_KEY
						)
		search_response = youtube.search().list(
			q=text,
			part="id,snippet",
			maxResults=50
		).execute()
		videos = []
		for search_result in search_response.get("items", []):
			if search_result["id"]["kind"] == "youtube#video":
				QCoreApplication.processEvents()
				title = search_result["snippet"]["title"]
				video_id = search_result["id"]["videoId"]
				url = "https://www.youtube.com/watch?v=" + video_id
				self.set_items(title, url)
				
	def set_items(self, title, url):
		"""Set items"""
		tableWidgetItem = QTableWidgetItem(QIcon(paths.yt_icon), title)
		tableWidgetItem.setFlags(tableWidgetItem.flags() ^ Qt.ItemIsEditable)
		tableWidgetItem.setWhatsThis(url)
		tableWidgetItem.setToolTip(title)
		#set row
		row = self.table_widget.rowCount()
		self.table_widget.insertRow(row)
		self.table_widget.setItem(row, 0, tableWidgetItem)
		
	def press_yt_item(self, tableWidgetItem):
		"""Press yt item"""
		url = tableWidgetItem.whatsThis()
		title = tableWidgetItem.text()
		self.parent().parent().parent().press_any_item(title, url)
		
	def press_add_pls_button(self):
		"""Press add all items to PLS"""
		if self.table_widget.rowCount() > 0:
			self.parent().parent().parent().pls_tab.clear()
			for i in range(0, self.table_widget.rowCount()):
				current_yt_item = self.table_widget.item(i,0)
				name = current_yt_item.text()
				url = current_yt_item.whatsThis()
				self.parent().parent().parent().pls_tab.set_row(name, url)
			self.parent().parent().parent().tabber.setCurrentIndex(1)
			
	def press_add_item_button(self):
		"""Add one item to PLS"""
		current_item = self.table_widget.currentItem()
		if current_item:
			title = current_item.text()
			url = current_item.whatsThis()
			self.parent().parent().parent().pls_tab.set_row(title, url)
	
	def press_add_fav_button(self):
		"""Add item to FAV"""
		current_item = self.table_widget.currentItem()
		if current_item:
			title = current_item.text()
			url = current_item.whatsThis()
			self.parent().parent().parent().open_add_win(title, url)
